"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def create(
    hub,
    ctx,
    cache_cluster_id: str,
    replication_group_id: str = None,
    az_mode: str = None,
    preferred_availability_zone: str = None,
    preferred_availability_zones: List = None,
    num_cache_nodes: int = None,
    cache_node_type: str = None,
    engine: str = None,
    engine_version: str = None,
    cache_parameter_group_name: str = None,
    cache_subnet_group_name: str = None,
    cache_security_group_names: List = None,
    security_group_ids: List = None,
    tags: List = None,
    snapshot_arns: List = None,
    snapshot_name: str = None,
    preferred_maintenance_window: str = None,
    port: int = None,
    notification_topic_arn: str = None,
    auto_minor_version_upgrade: bool = None,
    snapshot_retention_limit: int = None,
    snapshot_window: str = None,
    auth_token: str = None,
    outpost_mode: str = None,
    preferred_outpost_arn: str = None,
    preferred_outpost_arns: List = None,
    log_delivery_configurations: List = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Creates a cluster. All nodes in the cluster run the same protocol-compliant cache engine software, either Memcached or Redis. This operation is not supported for Redis (cluster mode enabled) clusters.

    Args:
        cache_cluster_id(str): The node group (shard) identifier. This parameter is stored as a lowercase string.  Constraints:    A name must contain from 1 to 50 alphanumeric characters or hyphens.   The first character must be a letter.   A name cannot end with a hyphen or contain two consecutive hyphens.  .
        replication_group_id(str, optional): The ID of the replication group to which this cluster should belong. If this parameter is specified, the cluster is added to the specified replication group as a read replica; otherwise, the cluster is a standalone primary that is not part of any replication group. If the specified replication group is Multi-AZ enabled and the Availability Zone is not specified, the cluster is created in Availability Zones that provide the best spread of read replicas across Availability Zones.  This parameter is only valid if the Engine parameter is redis. . Defaults to None.
        az_mode(str, optional): Specifies whether the nodes in this Memcached cluster are created in a single Availability Zone or created across multiple Availability Zones in the cluster's region. This parameter is only supported for Memcached clusters. If the AZMode and PreferredAvailabilityZones are not specified, ElastiCache assumes single-az mode. Defaults to None.
        preferred_availability_zone(str, optional): The EC2 Availability Zone in which the cluster is created. All nodes belonging to this cluster are placed in the preferred Availability Zone. If you want to create your nodes across multiple Availability Zones, use PreferredAvailabilityZones. Default: System chosen Availability Zone. Defaults to None.
        preferred_availability_zones(List, optional): A list of the Availability Zones in which cache nodes are created. The order of the zones in the list is not important. This option is only supported on Memcached.  If you are creating your cluster in an Amazon VPC (recommended) you can only locate nodes in Availability Zones that are associated with the subnets in the selected subnet group. The number of Availability Zones listed must equal the value of NumCacheNodes.  If you want all the nodes in the same Availability Zone, use PreferredAvailabilityZone instead, or repeat the Availability Zone multiple times in the list. Default: System chosen Availability Zones. Defaults to None.
        num_cache_nodes(int, optional): The initial number of cache nodes that the cluster has. For clusters running Redis, this value must be 1. For clusters running Memcached, this value must be between 1 and 40. If you need more than 40 nodes for your Memcached cluster, please fill out the ElastiCache Limit Increase Request form at http://aws.amazon.com/contact-us/elasticache-node-limit-request/. Defaults to None.
        cache_node_type(str, optional): The compute and memory capacity of the nodes in the node group (shard). The following node types are supported by ElastiCache. Generally speaking, the current generation types provide more memory and computational power at lower cost when compared to their equivalent previous generation counterparts.   General purpose:   Current generation:   M6g node types (available only for Redis engine version 5.0.6 onward and for Memcached engine version 1.5.16 onward).  cache.m6g.large, cache.m6g.xlarge, cache.m6g.2xlarge, cache.m6g.4xlarge, cache.m6g.8xlarge, cache.m6g.12xlarge, cache.m6g.16xlarge   For region availability, see Supported Node Types    M5 node types: cache.m5.large, cache.m5.xlarge, cache.m5.2xlarge, cache.m5.4xlarge, cache.m5.12xlarge, cache.m5.24xlarge   M4 node types: cache.m4.large, cache.m4.xlarge, cache.m4.2xlarge, cache.m4.4xlarge, cache.m4.10xlarge   T3 node types: cache.t3.micro, cache.t3.small, cache.t3.medium   T2 node types: cache.t2.micro, cache.t2.small, cache.t2.medium    Previous generation: (not recommended)  T1 node types: cache.t1.micro   M1 node types: cache.m1.small, cache.m1.medium, cache.m1.large, cache.m1.xlarge   M3 node types: cache.m3.medium, cache.m3.large, cache.m3.xlarge, cache.m3.2xlarge      Compute optimized:   Previous generation: (not recommended)  C1 node types: cache.c1.xlarge      Memory optimized:   Current generation:   R6g node types (available only for Redis engine version 5.0.6 onward and for Memcached engine version 1.5.16 onward).  cache.r6g.large, cache.r6g.xlarge, cache.r6g.2xlarge, cache.r6g.4xlarge, cache.r6g.8xlarge, cache.r6g.12xlarge, cache.r6g.16xlarge   For region availability, see Supported Node Types    R5 node types: cache.r5.large, cache.r5.xlarge, cache.r5.2xlarge, cache.r5.4xlarge, cache.r5.12xlarge, cache.r5.24xlarge   R4 node types: cache.r4.large, cache.r4.xlarge, cache.r4.2xlarge, cache.r4.4xlarge, cache.r4.8xlarge, cache.r4.16xlarge    Previous generation: (not recommended)  M2 node types: cache.m2.xlarge, cache.m2.2xlarge, cache.m2.4xlarge   R3 node types: cache.r3.large, cache.r3.xlarge, cache.r3.2xlarge, cache.r3.4xlarge, cache.r3.8xlarge       Additional node type info    All current generation instance types are created in Amazon VPC by default.   Redis append-only files (AOF) are not supported for T1 or T2 instances.   Redis Multi-AZ with automatic failover is not supported on T1 instances.   Redis configuration variables appendonly and appendfsync are not supported on Redis version 2.8.22 and later.  . Defaults to None.
        engine(str, optional): The name of the cache engine to be used for this cluster. Valid values for this parameter are: memcached | redis . Defaults to None.
        engine_version(str, optional): The version number of the cache engine to be used for this cluster. To view the supported cache engine versions, use the DescribeCacheEngineVersions operation.  Important: You can upgrade to a newer engine version (see Selecting a Cache Engine and Version), but you cannot downgrade to an earlier engine version. If you want to use an earlier engine version, you must delete the existing cluster or replication group and create it anew with the earlier engine version. . Defaults to None.
        cache_parameter_group_name(str, optional): The name of the parameter group to associate with this cluster. If this argument is omitted, the default parameter group for the specified engine is used. You cannot use any parameter group which has cluster-enabled='yes' when creating a cluster. Defaults to None.
        cache_subnet_group_name(str, optional): The name of the subnet group to be used for the cluster. Use this parameter only when you are creating a cluster in an Amazon Virtual Private Cloud (Amazon VPC).  If you're going to launch your cluster in an Amazon VPC, you need to create a subnet group before you start creating a cluster. For more information, see Subnets and Subnet Groups. . Defaults to None.
        cache_security_group_names(List, optional): A list of security group names to associate with this cluster. Use this parameter only when you are creating a cluster outside of an Amazon Virtual Private Cloud (Amazon VPC). Defaults to None.
        security_group_ids(List, optional): One or more VPC security groups associated with the cluster. Use this parameter only when you are creating a cluster in an Amazon Virtual Private Cloud (Amazon VPC). Defaults to None.
        tags(List, optional): A list of tags to be added to this resource. Defaults to None.
        snapshot_arns(List, optional): A single-element string list containing an Amazon Resource Name (ARN) that uniquely identifies a Redis RDB snapshot file stored in Amazon S3. The snapshot file is used to populate the node group (shard). The Amazon S3 object name in the ARN cannot contain any commas.  This parameter is only valid if the Engine parameter is redis.  Example of an Amazon S3 ARN: arn:aws:s3:::my_bucket/snapshot1.rdb . Defaults to None.
        snapshot_name(str, optional): The name of a Redis snapshot from which to restore data into the new node group (shard). The snapshot status changes to restoring while the new node group (shard) is being created.  This parameter is only valid if the Engine parameter is redis. . Defaults to None.
        preferred_maintenance_window(str, optional): Specifies the weekly time range during which maintenance on the cluster is performed. It is specified as a range in the format ddd:hh24:mi-ddd:hh24:mi (24H Clock UTC). The minimum maintenance window is a 60 minute period. Valid values for ddd are:. Defaults to None.
        port(int, optional): The port number on which each of the cache nodes accepts connections. Defaults to None.
        notification_topic_arn(str, optional): The Amazon Resource Name (ARN) of the Amazon Simple Notification Service (SNS) topic to which notifications are sent.  The Amazon SNS topic owner must be the same as the cluster owner. . Defaults to None.
        auto_minor_version_upgrade(bool, optional): This parameter is currently disabled. Defaults to None.
        snapshot_retention_limit(int, optional): The number of days for which ElastiCache retains automatic snapshots before deleting them. For example, if you set SnapshotRetentionLimit to 5, a snapshot taken today is retained for 5 days before being deleted.  This parameter is only valid if the Engine parameter is redis.  Default: 0 (i.e., automatic backups are disabled for this cache cluster). Defaults to None.
        snapshot_window(str, optional): The daily time range (in UTC) during which ElastiCache begins taking a daily snapshot of your node group (shard). Example: 05:00-09:00  If you do not specify this parameter, ElastiCache automatically chooses an appropriate time range.  This parameter is only valid if the Engine parameter is redis. . Defaults to None.
        auth_token(str, optional):  Reserved parameter. The password used to access a password protected server. Password constraints:   Must be only printable ASCII characters.   Must be at least 16 characters and no more than 128 characters in length.   The only permitted printable special characters are !, &, #, $, ^, <, >, and -. Other printable special characters cannot be used in the AUTH token.   For more information, see AUTH password at http://redis.io/commands/AUTH. Defaults to None.
        outpost_mode(str, optional): Specifies whether the nodes in the cluster are created in a single outpost or across multiple outposts. Defaults to None.
        preferred_outpost_arn(str, optional): The outpost ARN in which the cache cluster is created. Defaults to None.
        preferred_outpost_arns(List, optional): The outpost ARNs in which the cache cluster is created. Defaults to None.
        log_delivery_configurations(List, optional): Specifies the destination, format and type of the logs. . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.elasticache.cache.cluster.create(ctx, cache_cluster_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.elasticache.cache.cluster.create cache_cluster_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="elasticache",
        operation="create_cache_cluster",
        op_kwargs={
            "CacheClusterId": cache_cluster_id,
            "ReplicationGroupId": replication_group_id,
            "AZMode": az_mode,
            "PreferredAvailabilityZone": preferred_availability_zone,
            "PreferredAvailabilityZones": preferred_availability_zones,
            "NumCacheNodes": num_cache_nodes,
            "CacheNodeType": cache_node_type,
            "Engine": engine,
            "EngineVersion": engine_version,
            "CacheParameterGroupName": cache_parameter_group_name,
            "CacheSubnetGroupName": cache_subnet_group_name,
            "CacheSecurityGroupNames": cache_security_group_names,
            "SecurityGroupIds": security_group_ids,
            "Tags": tags,
            "SnapshotArns": snapshot_arns,
            "SnapshotName": snapshot_name,
            "PreferredMaintenanceWindow": preferred_maintenance_window,
            "Port": port,
            "NotificationTopicArn": notification_topic_arn,
            "AutoMinorVersionUpgrade": auto_minor_version_upgrade,
            "SnapshotRetentionLimit": snapshot_retention_limit,
            "SnapshotWindow": snapshot_window,
            "AuthToken": auth_token,
            "OutpostMode": outpost_mode,
            "PreferredOutpostArn": preferred_outpost_arn,
            "PreferredOutpostArns": preferred_outpost_arns,
            "LogDeliveryConfigurations": log_delivery_configurations,
        },
    )


async def delete(
    hub, ctx, cache_cluster_id: str, final_snapshot_identifier: str = None
) -> Dict:
    r"""
    **Autogenerated function**

    Deletes a previously provisioned cluster. DeleteCacheCluster deletes all associated cache nodes, node endpoints and the cluster itself. When you receive a successful response from this operation, Amazon ElastiCache immediately begins deleting the cluster; you cannot cancel or revert this operation. This operation is not valid for:   Redis (cluster mode enabled) clusters   Redis (cluster mode disabled) clusters   A cluster that is the last read replica of a replication group   A cluster that is the primary node of a replication group   A node group (shard) that has Multi-AZ mode enabled   A cluster from a Redis (cluster mode enabled) replication group   A cluster that is not in the available state

    Args:
        cache_cluster_id(str): The cluster identifier for the cluster to be deleted. This parameter is not case sensitive.
        final_snapshot_identifier(str, optional): The user-supplied name of a final cluster snapshot. This is the unique name that identifies the snapshot. ElastiCache creates the snapshot, and then deletes the cluster immediately afterward. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.elasticache.cache.cluster.delete(ctx, cache_cluster_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.elasticache.cache.cluster.delete cache_cluster_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="elasticache",
        operation="delete_cache_cluster",
        op_kwargs={
            "CacheClusterId": cache_cluster_id,
            "FinalSnapshotIdentifier": final_snapshot_identifier,
        },
    )


async def describe_all(
    hub,
    ctx,
    cache_cluster_id: str = None,
    max_records: int = None,
    marker: str = None,
    show_cache_node_info: bool = None,
    show_cache_clusters_not_in_replication_groups: bool = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Returns information about all provisioned clusters if no cluster identifier is specified, or about a specific cache cluster if a cluster identifier is supplied. By default, abbreviated information about the clusters is returned. You can use the optional ShowCacheNodeInfo flag to retrieve detailed information about the cache nodes associated with the clusters. These details include the DNS address and port for the cache node endpoint. If the cluster is in the creating state, only cluster-level information is displayed until all of the nodes are successfully provisioned. If the cluster is in the deleting state, only cluster-level information is displayed. If cache nodes are currently being added to the cluster, node endpoint information and creation time for the additional nodes are not displayed until they are completely provisioned. When the cluster state is available, the cluster is ready for use. If cache nodes are currently being removed from the cluster, no endpoint information for the removed nodes is displayed.

    Args:
        cache_cluster_id(str, optional): The user-supplied cluster identifier. If this parameter is specified, only information about that specific cluster is returned. This parameter isn't case sensitive. Defaults to None.
        max_records(int, optional): The maximum number of records to include in the response. If more records exist than the specified MaxRecords value, a marker is included in the response so that the remaining results can be retrieved. Default: 100 Constraints: minimum 20; maximum 100. Defaults to None.
        marker(str, optional): An optional marker returned from a prior request. Use this marker for pagination of results from this operation. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by MaxRecords. Defaults to None.
        show_cache_node_info(bool, optional): An optional flag that can be included in the DescribeCacheCluster request to retrieve information about the individual cache nodes. Defaults to None.
        show_cache_clusters_not_in_replication_groups(bool, optional): An optional flag that can be included in the DescribeCacheCluster request to show only nodes (API/CLI: clusters) that are not members of a replication group. In practice, this mean Memcached and single node Redis clusters. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.elasticache.cache.cluster.describe_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.elasticache.cache.cluster.describe_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="elasticache",
        operation="describe_cache_clusters",
        op_kwargs={
            "CacheClusterId": cache_cluster_id,
            "MaxRecords": max_records,
            "Marker": marker,
            "ShowCacheNodeInfo": show_cache_node_info,
            "ShowCacheClustersNotInReplicationGroups": show_cache_clusters_not_in_replication_groups,
        },
    )


async def modify(
    hub,
    ctx,
    cache_cluster_id: str,
    num_cache_nodes: int = None,
    cache_node_ids_to_remove: List = None,
    az_mode: str = None,
    new_availability_zones: List = None,
    cache_security_group_names: List = None,
    security_group_ids: List = None,
    preferred_maintenance_window: str = None,
    notification_topic_arn: str = None,
    cache_parameter_group_name: str = None,
    notification_topic_status: str = None,
    apply_immediately: bool = None,
    engine_version: str = None,
    auto_minor_version_upgrade: bool = None,
    snapshot_retention_limit: int = None,
    snapshot_window: str = None,
    cache_node_type: str = None,
    auth_token: str = None,
    auth_token_update_strategy: str = None,
    log_delivery_configurations: List = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Modifies the settings for a cluster. You can use this operation to change one or more cluster configuration parameters by specifying the parameters and the new values.

    Args:
        cache_cluster_id(str): The cluster identifier. This value is stored as a lowercase string.
        num_cache_nodes(int, optional): The number of cache nodes that the cluster should have. If the value for NumCacheNodes is greater than the sum of the number of current cache nodes and the number of cache nodes pending creation (which may be zero), more nodes are added. If the value is less than the number of existing cache nodes, nodes are removed. If the value is equal to the number of current cache nodes, any pending add or remove requests are canceled. If you are removing cache nodes, you must use the CacheNodeIdsToRemove parameter to provide the IDs of the specific cache nodes to remove. For clusters running Redis, this value must be 1. For clusters running Memcached, this value must be between 1 and 40.  Adding or removing Memcached cache nodes can be applied immediately or as a pending operation (see ApplyImmediately). A pending operation to modify the number of cache nodes in a cluster during its maintenance window, whether by adding or removing nodes in accordance with the scale out architecture, is not queued. The customer's latest request to add or remove nodes to the cluster overrides any previous pending operations to modify the number of cache nodes in the cluster. For example, a request to remove 2 nodes would override a previous pending operation to remove 3 nodes. Similarly, a request to add 2 nodes would override a previous pending operation to remove 3 nodes and vice versa. As Memcached cache nodes may now be provisioned in different Availability Zones with flexible cache node placement, a request to add nodes does not automatically override a previous pending operation to add nodes. The customer can modify the previous pending operation to add more nodes or explicitly cancel the pending request and retry the new request. To cancel pending operations to modify the number of cache nodes in a cluster, use the ModifyCacheCluster request and set NumCacheNodes equal to the number of cache nodes currently in the cluster. . Defaults to None.
        cache_node_ids_to_remove(List, optional): A list of cache node IDs to be removed. A node ID is a numeric identifier (0001, 0002, etc.). This parameter is only valid when NumCacheNodes is less than the existing number of cache nodes. The number of cache node IDs supplied in this parameter must match the difference between the existing number of cache nodes in the cluster or pending cache nodes, whichever is greater, and the value of NumCacheNodes in the request. For example: If you have 3 active cache nodes, 7 pending cache nodes, and the number of cache nodes in this ModifyCacheCluster call is 5, you must list 2 (7 - 5) cache node IDs to remove. Defaults to None.
        az_mode(str, optional): Specifies whether the new nodes in this Memcached cluster are all created in a single Availability Zone or created across multiple Availability Zones. Valid values: single-az | cross-az. This option is only supported for Memcached clusters.  You cannot specify single-az if the Memcached cluster already has cache nodes in different Availability Zones. If cross-az is specified, existing Memcached nodes remain in their current Availability Zone. Only newly created nodes are located in different Availability Zones.  . Defaults to None.
        new_availability_zones(List, optional):  This option is only supported on Memcached clusters.  The list of Availability Zones where the new Memcached cache nodes are created. This parameter is only valid when NumCacheNodes in the request is greater than the sum of the number of active cache nodes and the number of cache nodes pending creation (which may be zero). The number of Availability Zones supplied in this list must match the cache nodes being added in this request. Scenarios:    Scenario 1: You have 3 active nodes and wish to add 2 nodes. Specify NumCacheNodes=5 (3 + 2) and optionally specify two Availability Zones for the two new nodes.    Scenario 2: You have 3 active nodes and 2 nodes pending creation (from the scenario 1 call) and want to add 1 more node. Specify NumCacheNodes=6 ((3 + 2) + 1) and optionally specify an Availability Zone for the new node.    Scenario 3: You want to cancel all pending operations. Specify NumCacheNodes=3 to cancel all pending operations.   The Availability Zone placement of nodes pending creation cannot be modified. If you wish to cancel any nodes pending creation, add 0 nodes by setting NumCacheNodes to the number of current nodes. If cross-az is specified, existing Memcached nodes remain in their current Availability Zone. Only newly created nodes can be located in different Availability Zones. For guidance on how to move existing Memcached nodes to different Availability Zones, see the Availability Zone Considerations section of Cache Node Considerations for Memcached.  Impact of new add/remove requests upon pending requests    Scenario-1   Pending Action: Delete   New Request: Delete   Result: The new delete, pending or immediate, replaces the pending delete.     Scenario-2   Pending Action: Delete   New Request: Create   Result: The new create, pending or immediate, replaces the pending delete.     Scenario-3   Pending Action: Create   New Request: Delete   Result: The new delete, pending or immediate, replaces the pending create.     Scenario-4   Pending Action: Create   New Request: Create   Result: The new create is added to the pending create.   Important: If the new create request is Apply Immediately - Yes, all creates are performed immediately. If the new create request is Apply Immediately - No, all creates are pending.     . Defaults to None.
        cache_security_group_names(List, optional): A list of cache security group names to authorize on this cluster. This change is asynchronously applied as soon as possible. You can use this parameter only with clusters that are created outside of an Amazon Virtual Private Cloud (Amazon VPC). Constraints: Must contain no more than 255 alphanumeric characters. Must not be "Default". Defaults to None.
        security_group_ids(List, optional): Specifies the VPC Security Groups associated with the cluster. This parameter can be used only with clusters that are created in an Amazon Virtual Private Cloud (Amazon VPC). Defaults to None.
        preferred_maintenance_window(str, optional): Specifies the weekly time range during which maintenance on the cluster is performed. It is specified as a range in the format ddd:hh24:mi-ddd:hh24:mi (24H Clock UTC). The minimum maintenance window is a 60 minute period. Valid values for ddd are:    sun     mon     tue     wed     thu     fri     sat    Example: sun:23:00-mon:01:30 . Defaults to None.
        notification_topic_arn(str, optional): The Amazon Resource Name (ARN) of the Amazon SNS topic to which notifications are sent.  The Amazon SNS topic owner must be same as the cluster owner. . Defaults to None.
        cache_parameter_group_name(str, optional): The name of the cache parameter group to apply to this cluster. This change is asynchronously applied as soon as possible for parameters when the ApplyImmediately parameter is specified as true for this request. Defaults to None.
        notification_topic_status(str, optional): The status of the Amazon SNS notification topic. Notifications are sent only if the status is active. Valid values: active | inactive . Defaults to None.
        apply_immediately(bool, optional): If true, this parameter causes the modifications in this request and any pending modifications to be applied, asynchronously and as soon as possible, regardless of the PreferredMaintenanceWindow setting for the cluster. If false, changes to the cluster are applied on the next maintenance reboot, or the next failure reboot, whichever occurs first.  If you perform a ModifyCacheCluster before a pending modification is applied, the pending modification is replaced by the newer modification.  Valid values: true | false  Default: false . Defaults to None.
        engine_version(str, optional): The upgraded version of the cache engine to be run on the cache nodes.  Important: You can upgrade to a newer engine version (see Selecting a Cache Engine and Version), but you cannot downgrade to an earlier engine version. If you want to use an earlier engine version, you must delete the existing cluster and create it anew with the earlier engine version. . Defaults to None.
        auto_minor_version_upgrade(bool, optional): This parameter is currently disabled. Defaults to None.
        snapshot_retention_limit(int, optional): The number of days for which ElastiCache retains automatic cluster snapshots before deleting them. For example, if you set SnapshotRetentionLimit to 5, a snapshot that was taken today is retained for 5 days before being deleted.  If the value of SnapshotRetentionLimit is set to zero (0), backups are turned off. . Defaults to None.
        snapshot_window(str, optional): The daily time range (in UTC) during which ElastiCache begins taking a daily snapshot of your cluster. . Defaults to None.
        cache_node_type(str, optional): A valid cache node type that you want to scale this cluster up to. Defaults to None.
        auth_token(str, optional): Reserved parameter. The password used to access a password protected server. This parameter must be specified with the auth-token-update parameter. Password constraints:   Must be only printable ASCII characters   Must be at least 16 characters and no more than 128 characters in length   Cannot contain any of the following characters: '/', '"', or '@', '%'    For more information, see AUTH password at AUTH. Defaults to None.
        auth_token_update_strategy(str, optional): Specifies the strategy to use to update the AUTH token. This parameter must be specified with the auth-token parameter. Possible values:   Rotate   Set    For more information, see Authenticating Users with Redis AUTH . Defaults to None.
        log_delivery_configurations(List, optional): Specifies the destination, format and type of the logs. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.elasticache.cache.cluster.modify(ctx, cache_cluster_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.elasticache.cache.cluster.modify cache_cluster_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="elasticache",
        operation="modify_cache_cluster",
        op_kwargs={
            "CacheClusterId": cache_cluster_id,
            "NumCacheNodes": num_cache_nodes,
            "CacheNodeIdsToRemove": cache_node_ids_to_remove,
            "AZMode": az_mode,
            "NewAvailabilityZones": new_availability_zones,
            "CacheSecurityGroupNames": cache_security_group_names,
            "SecurityGroupIds": security_group_ids,
            "PreferredMaintenanceWindow": preferred_maintenance_window,
            "NotificationTopicArn": notification_topic_arn,
            "CacheParameterGroupName": cache_parameter_group_name,
            "NotificationTopicStatus": notification_topic_status,
            "ApplyImmediately": apply_immediately,
            "EngineVersion": engine_version,
            "AutoMinorVersionUpgrade": auto_minor_version_upgrade,
            "SnapshotRetentionLimit": snapshot_retention_limit,
            "SnapshotWindow": snapshot_window,
            "CacheNodeType": cache_node_type,
            "AuthToken": auth_token,
            "AuthTokenUpdateStrategy": auth_token_update_strategy,
            "LogDeliveryConfigurations": log_delivery_configurations,
        },
    )


async def reboot(
    hub, ctx, cache_cluster_id: str, cache_node_ids_to_reboot: List
) -> Dict:
    r"""
    **Autogenerated function**

    Reboots some, or all, of the cache nodes within a provisioned cluster. This operation applies any modified cache parameter groups to the cluster. The reboot operation takes place as soon as possible, and results in a momentary outage to the cluster. During the reboot, the cluster status is set to REBOOTING. The reboot causes the contents of the cache (for each cache node being rebooted) to be lost. When the reboot is complete, a cluster event is created. Rebooting a cluster is currently supported on Memcached and Redis (cluster mode disabled) clusters. Rebooting is not supported on Redis (cluster mode enabled) clusters. If you make changes to parameters that require a Redis (cluster mode enabled) cluster reboot for the changes to be applied, see Rebooting a Cluster for an alternate process.

    Args:
        cache_cluster_id(str): The cluster identifier. This parameter is stored as a lowercase string.
        cache_node_ids_to_reboot(List): A list of cache node IDs to reboot. A node ID is a numeric identifier (0001, 0002, etc.). To reboot an entire cluster, specify all of the cache node IDs.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.elasticache.cache.cluster.reboot(
                ctx, cache_cluster_id=value, cache_node_ids_to_reboot=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.elasticache.cache.cluster.reboot cache_cluster_id=value, cache_node_ids_to_reboot=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="elasticache",
        operation="reboot_cache_cluster",
        op_kwargs={
            "CacheClusterId": cache_cluster_id,
            "CacheNodeIdsToReboot": cache_node_ids_to_reboot,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def delete(hub, ctx, asset_id: str, data_set_id: str, revision_id: str) -> None:
    r"""
    **Autogenerated function**

    This operation deletes an asset.

    Args:
        asset_id(str): The unique identifier for an asset.
        data_set_id(str): The unique identifier for a data set.
        revision_id(str): The unique identifier for a revision.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.dataexchange.asset.delete(
                ctx, asset_id=value, data_set_id=value, revision_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.dataexchange.asset.delete asset_id=value, data_set_id=value, revision_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="dataexchange",
        operation="delete_asset",
        op_kwargs={
            "AssetId": asset_id,
            "DataSetId": data_set_id,
            "RevisionId": revision_id,
        },
    )


async def get(hub, ctx, asset_id: str, data_set_id: str, revision_id: str) -> Dict:
    r"""
    **Autogenerated function**

    This operation returns information about an asset.

    Args:
        asset_id(str): The unique identifier for an asset.
        data_set_id(str): The unique identifier for a data set.
        revision_id(str): The unique identifier for a revision.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.dataexchange.asset.get(
                ctx, asset_id=value, data_set_id=value, revision_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.dataexchange.asset.get asset_id=value, data_set_id=value, revision_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="dataexchange",
        operation="get_asset",
        op_kwargs={
            "AssetId": asset_id,
            "DataSetId": data_set_id,
            "RevisionId": revision_id,
        },
    )


async def update(
    hub, ctx, asset_id: str, data_set_id: str, name: str, revision_id: str
) -> Dict:
    r"""
    **Autogenerated function**

    This operation updates an asset.

    Args:
        asset_id(str): The unique identifier for an asset.
        data_set_id(str): The unique identifier for a data set.
        name(str): The name of the asset. When importing from Amazon S3, the S3 object key is used as the asset name. When exporting to Amazon S3, the asset name is used as default target S3 object key.
        revision_id(str): The unique identifier for a revision.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.dataexchange.asset.update(
                ctx, asset_id=value, data_set_id=value, name=value, revision_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.dataexchange.asset.update asset_id=value, data_set_id=value, name=value, revision_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="dataexchange",
        operation="update_asset",
        op_kwargs={
            "AssetId": asset_id,
            "DataSetId": data_set_id,
            "Name": name,
            "RevisionId": revision_id,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


__virtualname__ = "web_acl_id"


__func_alias__ = {"list_": "list"}


async def list_(
    hub, ctx, web_acl_id: str, marker: str = None, max_items: str = None
) -> Dict:
    r"""
    **Autogenerated function**

    List the distributions that are associated with a specified WAF web ACL.

    Args:
        marker(str, optional): Use Marker and MaxItems to control pagination of results. If you have more than MaxItems distributions that satisfy the request, the response includes a NextMarker element. To get the next page of results, submit another request. For the value of Marker, specify the value of NextMarker from the last response. (For the first request, omit Marker.) . Defaults to None.
        max_items(str, optional): The maximum number of distributions that you want CloudFront to return in the response body. The maximum and default values are both 100. Defaults to None.
        web_acl_id(str): The ID of the WAF web ACL that you want to list the associated distributions. If you specify "null" for the ID, the request returns a list of the distributions that aren't associated with a web ACL.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cloudfront.distribution.by.web_acl_id_.list(
                ctx, web_acl_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cloudfront.distribution.by.web_acl_id.list web_acl_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cloudfront",
        operation="list_distributions_by_web_acl_id",
        op_kwargs={"Marker": marker, "MaxItems": max_items, "WebACLId": web_acl_id},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def merge(
    hub, ctx, pull_request_id: str, repository_name: str, source_commit_id: str = None
) -> Dict:
    r"""
    **Autogenerated function**

    Attempts to merge the source commit of a pull request into the specified destination branch for that pull request at the specified commit using the fast-forward merge strategy. If the merge is successful, it closes the pull request.

    Args:
        pull_request_id(str): The system-generated ID of the pull request. To get this ID, use ListPullRequests.
        repository_name(str): The name of the repository where the pull request was created.
        source_commit_id(str, optional): The full commit ID of the original or updated commit in the pull request source branch. Pass this value if you want an exception thrown if the current commit ID of the tip of the source branch does not match this commit ID. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.codecommit.pull_request.by.fast_forward.merge(
                ctx, pull_request_id=value, repository_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.codecommit.pull_request.by.fast_forward.merge pull_request_id=value, repository_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="codecommit",
        operation="merge_pull_request_by_fast_forward",
        op_kwargs={
            "pullRequestId": pull_request_id,
            "repositoryName": repository_name,
            "sourceCommitId": source_commit_id,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def create(
    hub,
    ctx,
    name: str,
    creator_request_id: str = None,
    description: str = None,
    tags: List = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Creates an HTTP namespace. Service instances registered using an HTTP namespace can be discovered using a DiscoverInstances request but can't be discovered using DNS. For the current quota on the number of namespaces that you can create using the same account, see Cloud Map quotas in the Cloud Map Developer Guide.

    Args:
        name(str): The name that you want to assign to this namespace.
        creator_request_id(str, optional): A unique string that identifies the request and that allows failed CreateHttpNamespace requests to be retried without the risk of running the operation twice. CreatorRequestId can be any unique string (for example, a date/time stamp). Defaults to None.
        description(str, optional): A description for the namespace. Defaults to None.
        tags(List, optional): The tags to add to the namespace. Each tag consists of a key and an optional value that you define. Tags keys can be up to 128 characters in length, and tag values can be up to 256 characters in length. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.servicediscovery.http_namespace.create(ctx, name=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.servicediscovery.http_namespace.create name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="servicediscovery",
        operation="create_http_namespace",
        op_kwargs={
            "Name": name,
            "CreatorRequestId": creator_request_id,
            "Description": description,
            "Tags": tags,
        },
    )


async def update(
    hub, ctx, id_: str, namespace: Dict, updater_request_id: str = None
) -> Dict:
    r"""
    **Autogenerated function**

    Updates an HTTP namespace.

    Args:
        id_(str): The ID of the namespace that you want to update.
        updater_request_id(str, optional): A unique string that identifies the request and that allows failed UpdateHttpNamespace requests to be retried without the risk of running the operation twice. UpdaterRequestId can be any unique string (for example, a date/timestamp). Defaults to None.
        namespace(Dict): Updated properties for the the HTTP namespace.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.servicediscovery.http_namespace.update(
                ctx, id_=value, namespace=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.servicediscovery.http_namespace.update id_=value, namespace=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="servicediscovery",
        operation="update_http_namespace",
        op_kwargs={
            "Id": id_,
            "UpdaterRequestId": updater_request_id,
            "Namespace": namespace,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def describe(hub, ctx, stack_name: str, logical_resource_id: str) -> Dict:
    r"""
    **Autogenerated function**

    Returns a description of the specified resource in the specified stack. For deleted stacks, DescribeStackResource returns resource information for up to 90 days after the stack has been deleted.

    Args:
        stack_name(str): The name or the unique stack ID that is associated with the stack, which are not always interchangeable:   Running stacks: You can specify either the stack's name or its unique stack ID.   Deleted stacks: You must specify the unique stack ID.   Default: There is no default value.
        logical_resource_id(str): The logical name of the resource as specified in the template. Default: There is no default value.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cloudformation.stack.resource.init.describe(
                ctx, stack_name=value, logical_resource_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cloudformation.stack.resource.init.describe stack_name=value, logical_resource_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cloudformation",
        operation="describe_stack_resource",
        op_kwargs={"StackName": stack_name, "LogicalResourceId": logical_resource_id},
    )


async def describe_all(
    hub,
    ctx,
    stack_name: str = None,
    logical_resource_id: str = None,
    physical_resource_id: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Returns Amazon Web Services resource descriptions for running and deleted stacks. If StackName is specified, all the associated resources that are part of the stack are returned. If PhysicalResourceId is specified, the associated resources of the stack that the resource belongs to are returned.  Only the first 100 resources will be returned. If your stack has more resources than this, you should use ListStackResources instead.  For deleted stacks, DescribeStackResources returns resource information for up to 90 days after the stack has been deleted. You must specify either StackName or PhysicalResourceId, but not both. In addition, you can specify LogicalResourceId to filter the returned result. For more information about resources, the LogicalResourceId and PhysicalResourceId, go to the CloudFormation User Guide.  A ValidationError is returned if you specify both StackName and PhysicalResourceId in the same request.

    Args:
        stack_name(str, optional): The name or the unique stack ID that is associated with the stack, which are not always interchangeable:   Running stacks: You can specify either the stack's name or its unique stack ID.   Deleted stacks: You must specify the unique stack ID.   Default: There is no default value. Required: Conditional. If you do not specify StackName, you must specify PhysicalResourceId. Defaults to None.
        logical_resource_id(str, optional): The logical name of the resource as specified in the template. Default: There is no default value. Defaults to None.
        physical_resource_id(str, optional): The name or unique identifier that corresponds to a physical instance ID of a resource supported by CloudFormation. For example, for an Amazon Elastic Compute Cloud (EC2) instance, PhysicalResourceId corresponds to the InstanceId. You can pass the EC2 InstanceId to DescribeStackResources to find which stack the instance belongs to and what other resources are part of the stack. Required: Conditional. If you do not specify PhysicalResourceId, you must specify StackName. Default: There is no default value. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cloudformation.stack.resource.init.describe_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cloudformation.stack.resource.init.describe_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cloudformation",
        operation="describe_stack_resources",
        op_kwargs={
            "StackName": stack_name,
            "LogicalResourceId": logical_resource_id,
            "PhysicalResourceId": physical_resource_id,
        },
    )


async def list_all(hub, ctx, stack_name: str, next_token: str = None) -> Dict:
    r"""
    **Autogenerated function**

    Returns descriptions of all resources of the specified stack. For deleted stacks, ListStackResources returns resource information for up to 90 days after the stack has been deleted.

    Args:
        stack_name(str): The name or the unique stack ID that is associated with the stack, which are not always interchangeable:   Running stacks: You can specify either the stack's name or its unique stack ID.   Deleted stacks: You must specify the unique stack ID.   Default: There is no default value.
        next_token(str, optional): A string that identifies the next page of stack resources that you want to retrieve. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cloudformation.stack.resource.init.list_all(
                ctx, stack_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cloudformation.stack.resource.init.list_all stack_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cloudformation",
        operation="list_stack_resources",
        op_kwargs={"StackName": stack_name, "NextToken": next_token},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def create(
    hub, ctx, account_id: str, budget_name: str, notification: Dict, subscriber: Dict
) -> Dict:
    r"""
    **Autogenerated function**

    Creates a subscriber. You must create the associated budget and notification before you create the subscriber.

    Args:
        account_id(str): The accountId that is associated with the budget that you want to create a subscriber for.
        budget_name(str): The name of the budget that you want to subscribe to. Budget names must be unique within an account.
        notification(Dict): The notification that you want to create a subscriber for.
        subscriber(Dict): The subscriber that you want to associate with a budget notification.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.budget.subscriber.init.create(
                ctx, account_id=value, budget_name=value, notification=value, subscriber=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.budget.subscriber.init.create account_id=value, budget_name=value, notification=value, subscriber=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="budgets",
        operation="create_subscriber",
        op_kwargs={
            "AccountId": account_id,
            "BudgetName": budget_name,
            "Notification": notification,
            "Subscriber": subscriber,
        },
    )


async def delete(
    hub, ctx, account_id: str, budget_name: str, notification: Dict, subscriber: Dict
) -> Dict:
    r"""
    **Autogenerated function**

    Deletes a subscriber.  Deleting the last subscriber to a notification also deletes the notification.

    Args:
        account_id(str): The accountId that is associated with the budget whose subscriber you want to delete.
        budget_name(str): The name of the budget whose subscriber you want to delete.
        notification(Dict): The notification whose subscriber you want to delete.
        subscriber(Dict): The subscriber that you want to delete.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.budget.subscriber.init.delete(
                ctx, account_id=value, budget_name=value, notification=value, subscriber=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.budget.subscriber.init.delete account_id=value, budget_name=value, notification=value, subscriber=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="budgets",
        operation="delete_subscriber",
        op_kwargs={
            "AccountId": account_id,
            "BudgetName": budget_name,
            "Notification": notification,
            "Subscriber": subscriber,
        },
    )


async def update(
    hub,
    ctx,
    account_id: str,
    budget_name: str,
    notification: Dict,
    old_subscriber: Dict,
    new_subscriber: Dict,
) -> Dict:
    r"""
    **Autogenerated function**

    Updates a subscriber.

    Args:
        account_id(str): The accountId that is associated with the budget whose subscriber you want to update.
        budget_name(str): The name of the budget whose subscriber you want to update.
        notification(Dict): The notification whose subscriber you want to update.
        old_subscriber(Dict): The previous subscriber that is associated with a budget notification.
        new_subscriber(Dict): The updated subscriber that is associated with a budget notification.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.budget.subscriber.init.update(
                ctx,
                account_id=value,
                budget_name=value,
                notification=value,
                old_subscriber=value,
                new_subscriber=value,
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.budget.subscriber.init.update account_id=value, budget_name=value, notification=value, old_subscriber=value, new_subscriber=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="budgets",
        operation="update_subscriber",
        op_kwargs={
            "AccountId": account_id,
            "BudgetName": budget_name,
            "Notification": notification,
            "OldSubscriber": old_subscriber,
            "NewSubscriber": new_subscriber,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def get(hub, ctx, catalog_name: str, database_name: str) -> Dict:
    r"""
    **Autogenerated function**

    Returns a database object for the specified database and data catalog.

    Args:
        catalog_name(str): The name of the data catalog that contains the database to return.
        database_name(str): The name of the database to return.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.athena.database.get(
                ctx, catalog_name=value, database_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.athena.database.get catalog_name=value, database_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="athena",
        operation="get_database",
        op_kwargs={"CatalogName": catalog_name, "DatabaseName": database_name},
    )


async def list_all(
    hub, ctx, catalog_name: str, next_token: str = None, max_results: int = None
) -> Dict:
    r"""
    **Autogenerated function**

    Lists the databases in the specified data catalog.

    Args:
        catalog_name(str): The name of the data catalog that contains the databases to return.
        next_token(str, optional): A token generated by the Athena service that specifies where to continue pagination if a previous request was truncated. To obtain the next set of pages, pass in the NextToken from the response object of the previous page call. Defaults to None.
        max_results(int, optional): Specifies the maximum number of results to return. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.athena.database.list_all(ctx, catalog_name=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.athena.database.list_all catalog_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="athena",
        operation="list_databases",
        op_kwargs={
            "CatalogName": catalog_name,
            "NextToken": next_token,
            "MaxResults": max_results,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def checkout(
    hub,
    ctx,
    product_sku: str,
    checkout_type: str,
    key_fingerprint: str,
    entitlements: List,
    client_token: str,
    beneficiary: str = None,
    node_id: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Checks out the specified license.

    Args:
        product_sku(str): Product SKU.
        checkout_type(str): Checkout type.
        key_fingerprint(str): Key fingerprint identifying the license.
        entitlements(List): License entitlements.
        client_token(str): Unique, case-sensitive identifier that you provide to ensure the idempotency of the request.
        beneficiary(str, optional): License beneficiary. Defaults to None.
        node_id(str, optional): Node ID. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.license_manager.license_.init.checkout(
                ctx,
                product_sku=value,
                checkout_type=value,
                key_fingerprint=value,
                entitlements=value,
                client_token=value,
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.license_manager.license_.init.checkout product_sku=value, checkout_type=value, key_fingerprint=value, entitlements=value, client_token=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="license-manager",
        operation="checkout_license",
        op_kwargs={
            "ProductSKU": product_sku,
            "CheckoutType": checkout_type,
            "KeyFingerprint": key_fingerprint,
            "Entitlements": entitlements,
            "ClientToken": client_token,
            "Beneficiary": beneficiary,
            "NodeId": node_id,
        },
    )


async def create(
    hub,
    ctx,
    license_name: str,
    product_name: str,
    product_sku: str,
    issuer: Dict,
    home_region: str,
    validity: Dict,
    entitlements: List,
    beneficiary: str,
    consumption_configuration: Dict,
    client_token: str,
    license_metadata: List = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Creates a license.

    Args:
        license_name(str): License name.
        product_name(str): Product name.
        product_sku(str): Product SKU.
        issuer(Dict): License issuer.
        home_region(str): Home Region for the license.
        validity(Dict): Date and time range during which the license is valid, in ISO8601-UTC format.
        entitlements(List): License entitlements.
        beneficiary(str): License beneficiary.
        consumption_configuration(Dict): Configuration for consumption of the license. Choose a provisional configuration for workloads running with continuous connectivity. Choose a borrow configuration for workloads with offline usage.
        license_metadata(List, optional): Information about the license. Defaults to None.
        client_token(str): Unique, case-sensitive identifier that you provide to ensure the idempotency of the request.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.license_manager.license_.init.create(
                ctx,
                license_name=value,
                product_name=value,
                product_sku=value,
                issuer=value,
                home_region=value,
                validity=value,
                entitlements=value,
                beneficiary=value,
                consumption_configuration=value,
                client_token=value,
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.license_manager.license_.init.create license_name=value, product_name=value, product_sku=value, issuer=value, home_region=value, validity=value, entitlements=value, beneficiary=value, consumption_configuration=value, client_token=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="license-manager",
        operation="create_license",
        op_kwargs={
            "LicenseName": license_name,
            "ProductName": product_name,
            "ProductSKU": product_sku,
            "Issuer": issuer,
            "HomeRegion": home_region,
            "Validity": validity,
            "Entitlements": entitlements,
            "Beneficiary": beneficiary,
            "ConsumptionConfiguration": consumption_configuration,
            "LicenseMetadata": license_metadata,
            "ClientToken": client_token,
        },
    )


async def delete(hub, ctx, license_arn: str, source_version: str) -> Dict:
    r"""
    **Autogenerated function**

    Deletes the specified license.

    Args:
        license_arn(str): Amazon Resource Name (ARN) of the license.
        source_version(str): Current version of the license.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.license_manager.license_.init.delete(
                ctx, license_arn=value, source_version=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.license_manager.license_.init.delete license_arn=value, source_version=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="license-manager",
        operation="delete_license",
        op_kwargs={"LicenseArn": license_arn, "SourceVersion": source_version},
    )


async def get(hub, ctx, license_arn: str, version: str = None) -> Dict:
    r"""
    **Autogenerated function**

    Gets detailed information about the specified license.

    Args:
        license_arn(str): Amazon Resource Name (ARN) of the license.
        version(str, optional): License version. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.license_manager.license_.init.get(ctx, license_arn=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.license_manager.license_.init.get license_arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="license-manager",
        operation="get_license",
        op_kwargs={"LicenseArn": license_arn, "Version": version},
    )


async def list_all(
    hub,
    ctx,
    license_arns: List = None,
    filters: List = None,
    next_token: str = None,
    max_results: int = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Lists the licenses for your account.

    Args:
        license_arns(List, optional): Amazon Resource Names (ARNs) of the licenses. Defaults to None.
        filters(List, optional): Filters to scope the results. The following filters are supported:    Beneficiary     ProductSKU     Fingerprint     Status   . Defaults to None.
        next_token(str, optional): Token for the next set of results. Defaults to None.
        max_results(int, optional): Maximum number of results to return in a single call. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.license_manager.license_.init.list_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.license_manager.license_.init.list_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="license-manager",
        operation="list_licenses",
        op_kwargs={
            "LicenseArns": license_arns,
            "Filters": filters,
            "NextToken": next_token,
            "MaxResults": max_results,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def test(hub, ctx, template_name: str, template_data: str) -> Dict:
    r"""
    **Autogenerated function**

    Creates a preview of the MIME content of an email when provided with a template and a set of replacement data. You can execute this operation no more than once per second.

    Args:
        template_name(str): The name of the template that you want to render.
        template_data(str): A list of replacement values to apply to the template. This parameter is a JSON object, typically consisting of key-value pairs in which the keys correspond to replacement tags in the email template.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.sesv2.render_email_template.test(
                ctx, template_name=value, template_data=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.sesv2.render_email_template.test template_name=value, template_data=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="sesv2",
        operation="test_render_email_template",
        op_kwargs={"TemplateName": template_name, "TemplateData": template_data},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def batch(
    hub,
    ctx,
    database_name: str,
    table_name: str,
    partitions_to_get: List,
    catalog_id: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Retrieves partitions in a batch request.

    Args:
        catalog_id(str, optional): The ID of the Data Catalog where the partitions in question reside. If none is supplied, the Amazon Web Services account ID is used by default. Defaults to None.
        database_name(str): The name of the catalog database where the partitions reside.
        table_name(str): The name of the partitions' table.
        partitions_to_get(List): A list of partition values identifying the partitions to retrieve.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.glue.get.partition.batch(
                ctx, database_name=value, table_name=value, partitions_to_get=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.glue.get.partition.batch database_name=value, table_name=value, partitions_to_get=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="glue",
        operation="batch_get_partition",
        op_kwargs={
            "CatalogId": catalog_id,
            "DatabaseName": database_name,
            "TableName": table_name,
            "PartitionsToGet": partitions_to_get,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def delete(hub, ctx, query_definition_id: str) -> Dict:
    r"""
    **Autogenerated function**

    Deletes a saved CloudWatch Logs Insights query definition. A query definition contains details about a saved CloudWatch Logs Insights query. Each DeleteQueryDefinition operation can delete one query definition. You must have the logs:DeleteQueryDefinition permission to be able to perform this operation.

    Args:
        query_definition_id(str): The ID of the query definition that you want to delete. You can use DescribeQueryDefinitions to retrieve the IDs of your saved query definitions.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.logs.query.definition.delete(ctx, query_definition_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.logs.query.definition.delete query_definition_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="logs",
        operation="delete_query_definition",
        op_kwargs={"queryDefinitionId": query_definition_id},
    )


async def describe_all(
    hub,
    ctx,
    query_definition_name_prefix: str = None,
    max_results: int = None,
    next_token: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    This operation returns a paginated list of your saved CloudWatch Logs Insights query definitions. You can use the queryDefinitionNamePrefix parameter to limit the results to only the query definitions that have names that start with a certain string.

    Args:
        query_definition_name_prefix(str, optional): Use this parameter to filter your results to only the query definitions that have names that start with the prefix you specify. Defaults to None.
        max_results(int, optional): Limits the number of returned query definitions to the specified number. Defaults to None.
        next_token(str, optional): The token for the next set of items to return. The token expires after 24 hours. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.logs.query.definition.describe_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.logs.query.definition.describe_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="logs",
        operation="describe_query_definitions",
        op_kwargs={
            "queryDefinitionNamePrefix": query_definition_name_prefix,
            "maxResults": max_results,
            "nextToken": next_token,
        },
    )


async def put(
    hub,
    ctx,
    name: str,
    query_string: str,
    query_definition_id: str = None,
    log_group_names: List = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Creates or updates a query definition for CloudWatch Logs Insights. For more information, see Analyzing Log Data with CloudWatch Logs Insights. To update a query definition, specify its queryDefinitionId in your request. The values of name, queryString, and logGroupNames are changed to the values that you specify in your update operation. No current values are retained from the current query definition. For example, if you update a current query definition that includes log groups, and you don't specify the logGroupNames parameter in your update operation, the query definition changes to contain no log groups. You must have the logs:PutQueryDefinition permission to be able to perform this operation.

    Args:
        name(str): A name for the query definition. If you are saving a lot of query definitions, we recommend that you name them so that you can easily find the ones you want by using the first part of the name as a filter in the queryDefinitionNamePrefix parameter of DescribeQueryDefinitions.
        query_definition_id(str, optional): If you are updating a query definition, use this parameter to specify the ID of the query definition that you want to update. You can use DescribeQueryDefinitions to retrieve the IDs of your saved query definitions. If you are creating a query definition, do not specify this parameter. CloudWatch generates a unique ID for the new query definition and include it in the response to this operation. Defaults to None.
        log_group_names(List, optional): Use this parameter to include specific log groups as part of your query definition. If you are updating a query definition and you omit this parameter, then the updated definition will contain no log groups. Defaults to None.
        query_string(str): The query string to use for this definition. For more information, see CloudWatch Logs Insights Query Syntax.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.logs.query.definition.put(ctx, name=value, query_string=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.logs.query.definition.put name=value, query_string=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="logs",
        operation="put_query_definition",
        op_kwargs={
            "name": name,
            "queryDefinitionId": query_definition_id,
            "logGroupNames": log_group_names,
            "queryString": query_string,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def list_all(
    hub,
    ctx,
    resource_share_invitation_arn: str,
    next_token: str = None,
    max_results: int = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Lists the resources in a resource share that is shared with you but that the invitation is still pending for.

    Args:
        resource_share_invitation_arn(str): The Amazon Resource Name (ARN) of the invitation.
        next_token(str, optional): The token for the next page of results. Defaults to None.
        max_results(int, optional): The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned nextToken value. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ram.pending_invitation_resource.list_all(
                ctx, resource_share_invitation_arn=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ram.pending_invitation_resource.list_all resource_share_invitation_arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ram",
        operation="list_pending_invitation_resources",
        op_kwargs={
            "resourceShareInvitationArn": resource_share_invitation_arn,
            "nextToken": next_token,
            "maxResults": max_results,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def tag(hub, ctx, resource_arn: str, tags: Dict) -> Dict:
    r"""
    **Autogenerated function**

    Assigns tags to resources. A tag is a label that you assign to an AWS resource. Each tag consists of a key and an optional value, both of which you define. Tags enable you to categorize your AWS resources by attributes such as purpose, owner, or environment. When you have many resources of the same type, you can quickly identify a specific resource based on the tags you've assigned to it. For example, you can define a set of tags for your Amazon EMR on EKS clusters to help you track each cluster's owner and stack level. We recommend that you devise a consistent set of tag keys for each resource type. You can then search and filter the resources based on the tags that you add.

    Args:
        resource_arn(str): The ARN of resources.
        tags(Dict): The tags assigned to resources.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.emr_container.resource.tag(ctx, resource_arn=value, tags=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.emr_container.resource.tag resource_arn=value, tags=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="emr-containers",
        operation="tag_resource",
        op_kwargs={"resourceArn": resource_arn, "tags": tags},
    )


async def untag(hub, ctx, resource_arn: str, tag_keys: List) -> Dict:
    r"""
    **Autogenerated function**

    Removes tags from resources.

    Args:
        resource_arn(str): The ARN of resources.
        tag_keys(List): The tag keys of the resources.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.emr_container.resource.untag(
                ctx, resource_arn=value, tag_keys=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.emr_container.resource.untag resource_arn=value, tag_keys=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="emr-containers",
        operation="untag_resource",
        op_kwargs={"resourceArn": resource_arn, "tagKeys": tag_keys},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


__func_alias__ = {"set_": "set"}


async def set_(hub, ctx, identity: str, dkim_enabled: bool) -> Dict:
    r"""
    **Autogenerated function**

    Enables or disables Easy DKIM signing of email sent from an identity. If Easy DKIM signing is enabled for a domain, then Amazon SES uses DKIM to sign all email that it sends from addresses on that domain. If Easy DKIM signing is enabled for an email address, then Amazon SES uses DKIM to sign all email it sends from that address.  For email addresses (for example, user@example.com), you can only enable DKIM signing if the corresponding domain (in this case, example.com) has been set up to use Easy DKIM.  You can enable DKIM signing for an identity at any time after you start the verification process for the identity, even if the verification process isn't complete.  You can execute this operation no more than once per second. For more information about Easy DKIM signing, go to the Amazon SES Developer Guide.

    Args:
        identity(str): The identity for which DKIM signing should be enabled or disabled.
        dkim_enabled(bool): Sets whether DKIM signing is enabled for an identity. Set to true to enable DKIM signing for this identity; false to disable it. .

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ses.identity.dkim.enabled.set(
                ctx, identity=value, dkim_enabled=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ses.identity.dkim.enabled.set identity=value, dkim_enabled=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ses",
        operation="set_identity_dkim_enabled",
        op_kwargs={"Identity": identity, "DkimEnabled": dkim_enabled},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def delete(
    hub, ctx, requester_account_id: str, requester_aws_region: str
) -> None:
    r"""
    **Autogenerated function**

    Deletes pending authorization requests for a specified aggregator account in a specified region.

    Args:
        requester_account_id(str): The 12-digit account ID of the account requesting to aggregate data.
        requester_aws_region(str): The region requesting to aggregate data.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.config.pending_aggregation_request.delete(
                ctx, requester_account_id=value, requester_aws_region=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.config.pending_aggregation_request.delete requester_account_id=value, requester_aws_region=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="config",
        operation="delete_pending_aggregation_request",
        op_kwargs={
            "RequesterAccountId": requester_account_id,
            "RequesterAwsRegion": requester_aws_region,
        },
    )


async def describe_all(hub, ctx, limit: int = None, next_token: str = None) -> Dict:
    r"""
    **Autogenerated function**

    Returns a list of all pending aggregation requests.

    Args:
        limit(int, optional): The maximum number of evaluation results returned on each page. The default is maximum. If you specify 0, AWS Config uses the default. Defaults to None.
        next_token(str, optional): The nextToken string returned on a previous page that you use to get the next page of results in a paginated response. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.config.pending_aggregation_request.describe_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.config.pending_aggregation_request.describe_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="config",
        operation="describe_pending_aggregation_requests",
        op_kwargs={"Limit": limit, "NextToken": next_token},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def create(
    hub,
    ctx,
    connection_name: str,
    provider_type: str = None,
    tags: List = None,
    host_arn: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Creates a connection that can then be given to other AWS services like CodePipeline so that it can access third-party code repositories. The connection is in pending status until the third-party connection handshake is completed from the console.

    Args:
        provider_type(str, optional): The name of the external provider where your third-party code repository is configured. Defaults to None.
        connection_name(str): The name of the connection to be created. The name must be unique in the calling AWS account.
        tags(List, optional): The key-value pair to use when tagging the resource. Defaults to None.
        host_arn(str, optional): The Amazon Resource Name (ARN) of the host associated with the connection to be created. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.codestar_connection.connection.create(
                ctx, connection_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.codestar_connection.connection.create connection_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="codestar-connections",
        operation="create_connection",
        op_kwargs={
            "ProviderType": provider_type,
            "ConnectionName": connection_name,
            "Tags": tags,
            "HostArn": host_arn,
        },
    )


async def delete(hub, ctx, connection_arn: str) -> Dict:
    r"""
    **Autogenerated function**

    The connection to be deleted.

    Args:
        connection_arn(str): The Amazon Resource Name (ARN) of the connection to be deleted.  The ARN is never reused if the connection is deleted. .

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.codestar_connection.connection.delete(ctx, connection_arn=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.codestar_connection.connection.delete connection_arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="codestar-connections",
        operation="delete_connection",
        op_kwargs={"ConnectionArn": connection_arn},
    )


async def get(hub, ctx, connection_arn: str) -> Dict:
    r"""
    **Autogenerated function**

    Returns the connection ARN and details such as status, owner, and provider type.

    Args:
        connection_arn(str): The Amazon Resource Name (ARN) of a connection.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.codestar_connection.connection.get(ctx, connection_arn=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.codestar_connection.connection.get connection_arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="codestar-connections",
        operation="get_connection",
        op_kwargs={"ConnectionArn": connection_arn},
    )


async def list_all(
    hub,
    ctx,
    provider_type_filter: str = None,
    host_arn_filter: str = None,
    max_results: int = None,
    next_token: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Lists the connections associated with your account.

    Args:
        provider_type_filter(str, optional): Filters the list of connections to those associated with a specified provider, such as Bitbucket. Defaults to None.
        host_arn_filter(str, optional): Filters the list of connections to those associated with a specified host. Defaults to None.
        max_results(int, optional): The maximum number of results to return in a single call. To retrieve the remaining results, make another call with the returned nextToken value. Defaults to None.
        next_token(str, optional): The token that was returned from the previous ListConnections call, which can be used to return the next set of connections in the list. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.codestar_connection.connection.list_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.codestar_connection.connection.list_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="codestar-connections",
        operation="list_connections",
        op_kwargs={
            "ProviderTypeFilter": provider_type_filter,
            "HostArnFilter": host_arn_filter,
            "MaxResults": max_results,
            "NextToken": next_token,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def create(
    hub, ctx, load_balancer_name: str, policy_name: str, cookie_name: str
) -> Dict:
    r"""
    **Autogenerated function**

    Generates a stickiness policy with sticky session lifetimes that follow that of an application-generated cookie. This policy can be associated only with HTTP/HTTPS listeners. This policy is similar to the policy created by CreateLBCookieStickinessPolicy, except that the lifetime of the special Elastic Load Balancing cookie, AWSELB, follows the lifetime of the application-generated cookie specified in the policy configuration. The load balancer only inserts a new stickiness cookie when the application response includes a new application cookie. If the application cookie is explicitly removed or expires, the session stops being sticky until a new application cookie is issued. For more information, see Application-Controlled Session Stickiness in the Classic Load Balancers Guide.

    Args:
        load_balancer_name(str): The name of the load balancer.
        policy_name(str): The name of the policy being created. Policy names must consist of alphanumeric characters and dashes (-). This name must be unique within the set of policies for this load balancer.
        cookie_name(str): The name of the application cookie used for stickiness.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.elb.app_cookie_stickiness_policy.create(
                ctx, load_balancer_name=value, policy_name=value, cookie_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.elb.app_cookie_stickiness_policy.create load_balancer_name=value, policy_name=value, cookie_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="elb",
        operation="create_app_cookie_stickiness_policy",
        op_kwargs={
            "LoadBalancerName": load_balancer_name,
            "PolicyName": policy_name,
            "CookieName": cookie_name,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def get_all(hub, ctx, query_id: str) -> Dict:
    r"""
    **Autogenerated function**

    Returns the results from the specified query. Only the fields requested in the query are returned, along with a @ptr field, which is the identifier for the log record. You can use the value of @ptr in a GetLogRecord operation to get the full log record.  GetQueryResults does not start a query execution. To run a query, use StartQuery. If the value of the Status field in the output is Running, this operation returns only partial results. If you see a value of Scheduled or Running for the status, you can retry the operation later to see the final results.

    Args:
        query_id(str): The ID number of the query.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.logs.query.result.get_all(ctx, query_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.logs.query.result.get_all query_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="logs",
        operation="get_query_results",
        op_kwargs={"queryId": query_id},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import List


async def batch(hub, ctx, domain_name: str, items: List) -> None:
    r"""
    **Autogenerated function**

     Performs multiple DeleteAttributes operations in a single call, which reduces round trips and latencies. This enables Amazon SimpleDB to optimize requests, which generally yields better throughput.   The following limitations are enforced for this operation:  1 MB request size 25 item limit per BatchDeleteAttributes operation

    Args:
        domain_name(str): The name of the domain in which the attributes are being deleted.
        items(List): A list of items on which to perform the operation.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.sdb.delete_attribute.batch(ctx, domain_name=value, items=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.sdb.delete_attribute.batch domain_name=value, items=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="sdb",
        operation="batch_delete_attributes",
        op_kwargs={"DomainName": domain_name, "Items": items},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def create(
    hub,
    ctx,
    endpoint_identifier: str,
    endpoint_type: str,
    engine_name: str,
    username: str = None,
    password: str = None,
    server_name: str = None,
    port: int = None,
    database_name: str = None,
    extra_connection_attributes: str = None,
    kms_key_id: str = None,
    tags: List = None,
    certificate_arn: str = None,
    ssl_mode: str = None,
    service_access_role_arn: str = None,
    external_table_definition: str = None,
    dynamo_db_settings: Dict = None,
    s3_settings: Dict = None,
    dms_transfer_settings: Dict = None,
    mongo_db_settings: Dict = None,
    kinesis_settings: Dict = None,
    kafka_settings: Dict = None,
    elasticsearch_settings: Dict = None,
    neptune_settings: Dict = None,
    redshift_settings: Dict = None,
    postgre_sql_settings: Dict = None,
    my_sql_settings: Dict = None,
    oracle_settings: Dict = None,
    sybase_settings: Dict = None,
    microsoft_sql_server_settings: Dict = None,
    ibm_db2_settings: Dict = None,
    resource_identifier: str = None,
    doc_db_settings: Dict = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Creates an endpoint using the provided settings.  For a MySQL source or target endpoint, don't explicitly specify the database using the DatabaseName request parameter on the CreateEndpoint API call. Specifying DatabaseName when you create a MySQL endpoint replicates all the task tables to this single database. For MySQL endpoints, you specify the database only when you specify the schema in the table-mapping rules of the DMS task.

    Args:
        endpoint_identifier(str): The database endpoint identifier. Identifiers must begin with a letter and must contain only ASCII letters, digits, and hyphens. They can't end with a hyphen, or contain two consecutive hyphens.
        endpoint_type(str): The type of endpoint. Valid values are source and target.
        engine_name(str): The type of engine for the endpoint. Valid values, depending on the EndpointType value, include "mysql", "oracle", "postgres", "mariadb", "aurora", "aurora-postgresql", "redshift", "s3", "db2", "azuredb", "sybase", "dynamodb", "mongodb", "kinesis", "kafka", "elasticsearch", "docdb", "sqlserver", and "neptune".
        username(str, optional): The user name to be used to log in to the endpoint database. Defaults to None.
        password(str, optional): The password to be used to log in to the endpoint database. Defaults to None.
        server_name(str, optional): The name of the server where the endpoint database resides. Defaults to None.
        port(int, optional): The port used by the endpoint database. Defaults to None.
        database_name(str, optional): The name of the endpoint database. For a MySQL source or target endpoint, do not specify DatabaseName. Defaults to None.
        extra_connection_attributes(str, optional): Additional attributes associated with the connection. Each attribute is specified as a name-value pair associated by an equal sign (=). Multiple attributes are separated by a semicolon (;) with no additional white space. For information on the attributes available for connecting your source or target endpoint, see Working with DMS Endpoints in the Database Migration Service User Guide. . Defaults to None.
        kms_key_id(str, optional): An KMS key identifier that is used to encrypt the connection parameters for the endpoint. If you don't specify a value for the KmsKeyId parameter, then DMS uses your default encryption key. KMS creates the default encryption key for your account. Your account has a different default encryption key for each Region. Defaults to None.
        tags(List, optional): One or more tags to be assigned to the endpoint. Defaults to None.
        certificate_arn(str, optional): The Amazon Resource Name (ARN) for the certificate. Defaults to None.
        ssl_mode(str, optional): The Secure Sockets Layer (SSL) mode to use for the SSL connection. The default is none . Defaults to None.
        service_access_role_arn(str, optional):  The Amazon Resource Name (ARN) for the service access role that you want to use to create the endpoint. The role must allow the iam:PassRole action. Defaults to None.
        external_table_definition(str, optional): The external table definition. . Defaults to None.
        dynamo_db_settings(Dict, optional): Settings in JSON format for the target Amazon DynamoDB endpoint. For information about other available settings, see Using Object Mapping to Migrate Data to DynamoDB in the Database Migration Service User Guide. . Defaults to None.
        s3_settings(Dict, optional): Settings in JSON format for the target Amazon S3 endpoint. For more information about the available settings, see Extra Connection Attributes When Using Amazon S3 as a Target for DMS in the Database Migration Service User Guide. . Defaults to None.
        dms_transfer_settings(Dict, optional): The settings in JSON format for the DMS transfer type of source endpoint.  Possible settings include the following:    ServiceAccessRoleArn - The IAM role that has permission to access the Amazon S3 bucket. The role must allow the iam:PassRole action.    BucketName - The name of the S3 bucket to use.   Shorthand syntax for these settings is as follows: ServiceAccessRoleArn=string,BucketName=string  JSON syntax for these settings is as follows: { "ServiceAccessRoleArn": "string", "BucketName": "string", }  . Defaults to None.
        mongo_db_settings(Dict, optional): Settings in JSON format for the source MongoDB endpoint. For more information about the available settings, see Endpoint configuration settings when using MongoDB as a source for Database Migration Service in the Database Migration Service User Guide. . Defaults to None.
        kinesis_settings(Dict, optional): Settings in JSON format for the target endpoint for Amazon Kinesis Data Streams. For more information about the available settings, see Using object mapping to migrate data to a Kinesis data stream in the Database Migration Service User Guide. . Defaults to None.
        kafka_settings(Dict, optional): Settings in JSON format for the target Apache Kafka endpoint. For more information about the available settings, see Using object mapping to migrate data to a Kafka topic in the Database Migration Service User Guide. . Defaults to None.
        elasticsearch_settings(Dict, optional): Settings in JSON format for the target Elasticsearch endpoint. For more information about the available settings, see Extra Connection Attributes When Using Elasticsearch as a Target for DMS in the Database Migration Service User Guide. Defaults to None.
        neptune_settings(Dict, optional): Settings in JSON format for the target Amazon Neptune endpoint. For more information about the available settings, see Specifying graph-mapping rules using Gremlin and R2RML for Amazon Neptune as a target in the Database Migration Service User Guide. . Defaults to None.
        redshift_settings(Dict, optional): Provides information that defines an Amazon Redshift endpoint. Defaults to None.
        postgre_sql_settings(Dict, optional): Settings in JSON format for the source and target PostgreSQL endpoint. For information about other available settings, see Extra connection attributes when using PostgreSQL as a source for DMS and  Extra connection attributes when using PostgreSQL as a target for DMS in the Database Migration Service User Guide. . Defaults to None.
        my_sql_settings(Dict, optional): Settings in JSON format for the source and target MySQL endpoint. For information about other available settings, see Extra connection attributes when using MySQL as a source for DMS and Extra connection attributes when using a MySQL-compatible database as a target for DMS in the Database Migration Service User Guide. . Defaults to None.
        oracle_settings(Dict, optional): Settings in JSON format for the source and target Oracle endpoint. For information about other available settings, see Extra connection attributes when using Oracle as a source for DMS and  Extra connection attributes when using Oracle as a target for DMS in the Database Migration Service User Guide. . Defaults to None.
        sybase_settings(Dict, optional): Settings in JSON format for the source and target SAP ASE endpoint. For information about other available settings, see Extra connection attributes when using SAP ASE as a source for DMS and Extra connection attributes when using SAP ASE as a target for DMS in the Database Migration Service User Guide. . Defaults to None.
        microsoft_sql_server_settings(Dict, optional): Settings in JSON format for the source and target Microsoft SQL Server endpoint. For information about other available settings, see Extra connection attributes when using SQL Server as a source for DMS and  Extra connection attributes when using SQL Server as a target for DMS in the Database Migration Service User Guide. . Defaults to None.
        ibm_db2_settings(Dict, optional): Settings in JSON format for the source IBM Db2 LUW endpoint. For information about other available settings, see Extra connection attributes when using Db2 LUW as a source for DMS in the Database Migration Service User Guide. . Defaults to None.
        resource_identifier(str, optional): A friendly name for the resource identifier at the end of the EndpointArn response parameter that is returned in the created Endpoint object. The value for this parameter can have up to 31 characters. It can contain only ASCII letters, digits, and hyphen ('-'). Also, it can't end with a hyphen or contain two consecutive hyphens, and can only begin with a letter, such as Example-App-ARN1. For example, this value might result in the EndpointArn value arn:aws:dms:eu-west-1:012345678901:rep:Example-App-ARN1. If you don't specify a ResourceIdentifier value, DMS generates a default identifier value for the end of EndpointArn. Defaults to None.
        doc_db_settings(Dict, optional): Provides information that defines a DocumentDB endpoint. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.dms.endpoint.init.create(
                ctx, endpoint_identifier=value, endpoint_type=value, engine_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.dms.endpoint.init.create endpoint_identifier=value, endpoint_type=value, engine_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="dms",
        operation="create_endpoint",
        op_kwargs={
            "EndpointIdentifier": endpoint_identifier,
            "EndpointType": endpoint_type,
            "EngineName": engine_name,
            "Username": username,
            "Password": password,
            "ServerName": server_name,
            "Port": port,
            "DatabaseName": database_name,
            "ExtraConnectionAttributes": extra_connection_attributes,
            "KmsKeyId": kms_key_id,
            "Tags": tags,
            "CertificateArn": certificate_arn,
            "SslMode": ssl_mode,
            "ServiceAccessRoleArn": service_access_role_arn,
            "ExternalTableDefinition": external_table_definition,
            "DynamoDbSettings": dynamo_db_settings,
            "S3Settings": s3_settings,
            "DmsTransferSettings": dms_transfer_settings,
            "MongoDbSettings": mongo_db_settings,
            "KinesisSettings": kinesis_settings,
            "KafkaSettings": kafka_settings,
            "ElasticsearchSettings": elasticsearch_settings,
            "NeptuneSettings": neptune_settings,
            "RedshiftSettings": redshift_settings,
            "PostgreSQLSettings": postgre_sql_settings,
            "MySQLSettings": my_sql_settings,
            "OracleSettings": oracle_settings,
            "SybaseSettings": sybase_settings,
            "MicrosoftSQLServerSettings": microsoft_sql_server_settings,
            "IBMDb2Settings": ibm_db2_settings,
            "ResourceIdentifier": resource_identifier,
            "DocDbSettings": doc_db_settings,
        },
    )


async def delete(hub, ctx, endpoint_arn: str) -> Dict:
    r"""
    **Autogenerated function**

    Deletes the specified endpoint.  All tasks associated with the endpoint must be deleted before you can delete the endpoint.

    Args:
        endpoint_arn(str): The Amazon Resource Name (ARN) string that uniquely identifies the endpoint.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.dms.endpoint.init.delete(ctx, endpoint_arn=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.dms.endpoint.init.delete endpoint_arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="dms",
        operation="delete_endpoint",
        op_kwargs={"EndpointArn": endpoint_arn},
    )


async def describe_all(
    hub, ctx, filters: List = None, max_records: int = None, marker: str = None
) -> Dict:
    r"""
    **Autogenerated function**

    Returns information about the endpoints for your account in the current region.

    Args:
        filters(List, optional): Filters applied to the endpoints. Valid filter names: endpoint-arn | endpoint-type | endpoint-id | engine-name. Defaults to None.
        max_records(int, optional):  The maximum number of records to include in the response. If more records exist than the specified MaxRecords value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.  Default: 100 Constraints: Minimum 20, maximum 100. Defaults to None.
        marker(str, optional):  An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by MaxRecords. . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.dms.endpoint.init.describe_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.dms.endpoint.init.describe_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="dms",
        operation="describe_endpoints",
        op_kwargs={"Filters": filters, "MaxRecords": max_records, "Marker": marker},
    )


async def modify(
    hub,
    ctx,
    endpoint_arn: str,
    endpoint_identifier: str = None,
    endpoint_type: str = None,
    engine_name: str = None,
    username: str = None,
    password: str = None,
    server_name: str = None,
    port: int = None,
    database_name: str = None,
    extra_connection_attributes: str = None,
    certificate_arn: str = None,
    ssl_mode: str = None,
    service_access_role_arn: str = None,
    external_table_definition: str = None,
    dynamo_db_settings: Dict = None,
    s3_settings: Dict = None,
    dms_transfer_settings: Dict = None,
    mongo_db_settings: Dict = None,
    kinesis_settings: Dict = None,
    kafka_settings: Dict = None,
    elasticsearch_settings: Dict = None,
    neptune_settings: Dict = None,
    redshift_settings: Dict = None,
    postgre_sql_settings: Dict = None,
    my_sql_settings: Dict = None,
    oracle_settings: Dict = None,
    sybase_settings: Dict = None,
    microsoft_sql_server_settings: Dict = None,
    ibm_db2_settings: Dict = None,
    doc_db_settings: Dict = None,
    exact_settings: bool = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Modifies the specified endpoint.  For a MySQL source or target endpoint, don't explicitly specify the database using the DatabaseName request parameter on the ModifyEndpoint API call. Specifying DatabaseName when you modify a MySQL endpoint replicates all the task tables to this single database. For MySQL endpoints, you specify the database only when you specify the schema in the table-mapping rules of the DMS task.

    Args:
        endpoint_arn(str): The Amazon Resource Name (ARN) string that uniquely identifies the endpoint.
        endpoint_identifier(str, optional): The database endpoint identifier. Identifiers must begin with a letter and must contain only ASCII letters, digits, and hyphens. They can't end with a hyphen or contain two consecutive hyphens. Defaults to None.
        endpoint_type(str, optional): The type of endpoint. Valid values are source and target. Defaults to None.
        engine_name(str, optional): The type of engine for the endpoint. Valid values, depending on the EndpointType, include "mysql", "oracle", "postgres", "mariadb", "aurora", "aurora-postgresql", "redshift", "s3", "db2", "azuredb", "sybase", "dynamodb", "mongodb", "kinesis", "kafka", "elasticsearch", "documentdb", "sqlserver", and "neptune". Defaults to None.
        username(str, optional): The user name to be used to login to the endpoint database. Defaults to None.
        password(str, optional): The password to be used to login to the endpoint database. Defaults to None.
        server_name(str, optional): The name of the server where the endpoint database resides. Defaults to None.
        port(int, optional): The port used by the endpoint database. Defaults to None.
        database_name(str, optional): The name of the endpoint database. For a MySQL source or target endpoint, do not specify DatabaseName. Defaults to None.
        extra_connection_attributes(str, optional): Additional attributes associated with the connection. To reset this parameter, pass the empty string ("") as an argument. Defaults to None.
        certificate_arn(str, optional): The Amazon Resource Name (ARN) of the certificate used for SSL connection. Defaults to None.
        ssl_mode(str, optional): The SSL mode used to connect to the endpoint. The default value is none. Defaults to None.
        service_access_role_arn(str, optional):  The Amazon Resource Name (ARN) for the IAM role you want to use to modify the endpoint. The role must allow the iam:PassRole action. Defaults to None.
        external_table_definition(str, optional): The external table definition. Defaults to None.
        dynamo_db_settings(Dict, optional): Settings in JSON format for the target Amazon DynamoDB endpoint. For information about other available settings, see Using Object Mapping to Migrate Data to DynamoDB in the Database Migration Service User Guide. . Defaults to None.
        s3_settings(Dict, optional): Settings in JSON format for the target Amazon S3 endpoint. For more information about the available settings, see Extra Connection Attributes When Using Amazon S3 as a Target for DMS in the Database Migration Service User Guide. . Defaults to None.
        dms_transfer_settings(Dict, optional): The settings in JSON format for the DMS transfer type of source endpoint.  Attributes include the following:   serviceAccessRoleArn - The Identity and Access Management (IAM) role that has permission to access the Amazon S3 bucket. The role must allow the iam:PassRole action.   BucketName - The name of the S3 bucket to use.   Shorthand syntax for these settings is as follows: ServiceAccessRoleArn=string ,BucketName=string  JSON syntax for these settings is as follows: { "ServiceAccessRoleArn": "string", "BucketName": "string"}  . Defaults to None.
        mongo_db_settings(Dict, optional): Settings in JSON format for the source MongoDB endpoint. For more information about the available settings, see the configuration properties section in Endpoint configuration settings when using MongoDB as a source for Database Migration Service in the Database Migration Service User Guide. . Defaults to None.
        kinesis_settings(Dict, optional): Settings in JSON format for the target endpoint for Amazon Kinesis Data Streams. For more information about the available settings, see Using object mapping to migrate data to a Kinesis data stream in the Database Migration Service User Guide. . Defaults to None.
        kafka_settings(Dict, optional): Settings in JSON format for the target Apache Kafka endpoint. For more information about the available settings, see Using object mapping to migrate data to a Kafka topic in the Database Migration Service User Guide. . Defaults to None.
        elasticsearch_settings(Dict, optional): Settings in JSON format for the target Elasticsearch endpoint. For more information about the available settings, see Extra Connection Attributes When Using Elasticsearch as a Target for DMS in the Database Migration Service User Guide. . Defaults to None.
        neptune_settings(Dict, optional): Settings in JSON format for the target Amazon Neptune endpoint. For more information about the available settings, see Specifying graph-mapping rules using Gremlin and R2RML for Amazon Neptune as a target in the Database Migration Service User Guide. . Defaults to None.
        redshift_settings(Dict, optional): Provides information that defines an Amazon Redshift endpoint. Defaults to None.
        postgre_sql_settings(Dict, optional): Settings in JSON format for the source and target PostgreSQL endpoint. For information about other available settings, see Extra connection attributes when using PostgreSQL as a source for DMS and  Extra connection attributes when using PostgreSQL as a target for DMS in the Database Migration Service User Guide. . Defaults to None.
        my_sql_settings(Dict, optional): Settings in JSON format for the source and target MySQL endpoint. For information about other available settings, see Extra connection attributes when using MySQL as a source for DMS and Extra connection attributes when using a MySQL-compatible database as a target for DMS in the Database Migration Service User Guide. . Defaults to None.
        oracle_settings(Dict, optional): Settings in JSON format for the source and target Oracle endpoint. For information about other available settings, see Extra connection attributes when using Oracle as a source for DMS and  Extra connection attributes when using Oracle as a target for DMS in the Database Migration Service User Guide. . Defaults to None.
        sybase_settings(Dict, optional): Settings in JSON format for the source and target SAP ASE endpoint. For information about other available settings, see Extra connection attributes when using SAP ASE as a source for DMS and Extra connection attributes when using SAP ASE as a target for DMS in the Database Migration Service User Guide. . Defaults to None.
        microsoft_sql_server_settings(Dict, optional): Settings in JSON format for the source and target Microsoft SQL Server endpoint. For information about other available settings, see Extra connection attributes when using SQL Server as a source for DMS and  Extra connection attributes when using SQL Server as a target for DMS in the Database Migration Service User Guide. . Defaults to None.
        ibm_db2_settings(Dict, optional): Settings in JSON format for the source IBM Db2 LUW endpoint. For information about other available settings, see Extra connection attributes when using Db2 LUW as a source for DMS in the Database Migration Service User Guide. . Defaults to None.
        doc_db_settings(Dict, optional): Settings in JSON format for the source DocumentDB endpoint. For more information about the available settings, see the configuration properties section in  Using DocumentDB as a Target for Database Migration Service  in the Database Migration Service User Guide. . Defaults to None.
        exact_settings(bool, optional): If this attribute is Y, the current call to ModifyEndpoint replaces all existing endpoint settings with the exact settings that you specify in this call. If this attribute is N, the current call to ModifyEndpoint does two things:    It replaces any endpoint settings that already exist with new values, for settings with the same names.   It creates new endpoint settings that you specify in the call, for settings with different names.    For example, if you call create-endpoint ... --endpoint-settings '{"a":1}' ..., the endpoint has the following endpoint settings: '{"a":1}'. If you then call modify-endpoint ... --endpoint-settings '{"b":2}' ... for the same endpoint, the endpoint has the following settings: '{"a":1,"b":2}'.  However, suppose that you follow this with a call to modify-endpoint ... --endpoint-settings '{"b":2}' --exact-settings ... for that same endpoint again. Then the endpoint has the following settings: '{"b":2}'. All existing settings are replaced with the exact settings that you specify. . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.dms.endpoint.init.modify(ctx, endpoint_arn=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.dms.endpoint.init.modify endpoint_arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="dms",
        operation="modify_endpoint",
        op_kwargs={
            "EndpointArn": endpoint_arn,
            "EndpointIdentifier": endpoint_identifier,
            "EndpointType": endpoint_type,
            "EngineName": engine_name,
            "Username": username,
            "Password": password,
            "ServerName": server_name,
            "Port": port,
            "DatabaseName": database_name,
            "ExtraConnectionAttributes": extra_connection_attributes,
            "CertificateArn": certificate_arn,
            "SslMode": ssl_mode,
            "ServiceAccessRoleArn": service_access_role_arn,
            "ExternalTableDefinition": external_table_definition,
            "DynamoDbSettings": dynamo_db_settings,
            "S3Settings": s3_settings,
            "DmsTransferSettings": dms_transfer_settings,
            "MongoDbSettings": mongo_db_settings,
            "KinesisSettings": kinesis_settings,
            "KafkaSettings": kafka_settings,
            "ElasticsearchSettings": elasticsearch_settings,
            "NeptuneSettings": neptune_settings,
            "RedshiftSettings": redshift_settings,
            "PostgreSQLSettings": postgre_sql_settings,
            "MySQLSettings": my_sql_settings,
            "OracleSettings": oracle_settings,
            "SybaseSettings": sybase_settings,
            "MicrosoftSQLServerSettings": microsoft_sql_server_settings,
            "IBMDb2Settings": ibm_db2_settings,
            "DocDbSettings": doc_db_settings,
            "ExactSettings": exact_settings,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def delete(hub, ctx, resource_type: str, resource_id: str) -> None:
    r"""
    **Autogenerated function**

    Records the configuration state for a custom resource that has been deleted. This API records a new ConfigurationItem with a ResourceDeleted status. You can retrieve the ConfigurationItems recorded for this resource in your AWS Config History.

    Args:
        resource_type(str): The type of the resource.
        resource_id(str): Unique identifier of the resource.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.config.resource.config.init.delete(
                ctx, resource_type=value, resource_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.config.resource.config.init.delete resource_type=value, resource_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="config",
        operation="delete_resource_config",
        op_kwargs={"ResourceType": resource_type, "ResourceId": resource_id},
    )


async def put(
    hub,
    ctx,
    resource_type: str,
    schema_version_id: str,
    resource_id: str,
    configuration: str,
    resource_name: str = None,
    tags: Dict = None,
) -> None:
    r"""
    **Autogenerated function**

    Records the configuration state for the resource provided in the request. The configuration state of a resource is represented in AWS Config as Configuration Items. Once this API records the configuration item, you can retrieve the list of configuration items for the custom resource type using existing AWS Config APIs.   The custom resource type must be registered with AWS CloudFormation. This API accepts the configuration item registered with AWS CloudFormation. When you call this API, AWS Config only stores configuration state of the resource provided in the request. This API does not change or remediate the configuration of the resource.  Write-only schema properites are not recorded as part of the published configuration item.

    Args:
        resource_type(str): The type of the resource. The custom resource type must be registered with AWS CloudFormation.   You cannot use the organization names “aws”, “amzn”, “amazon”, “alexa”, “custom” with custom resource types. It is the first part of the ResourceType up to the first ::. .
        schema_version_id(str): Version of the schema registered for the ResourceType in AWS CloudFormation.
        resource_id(str): Unique identifier of the resource.
        resource_name(str, optional): Name of the resource. Defaults to None.
        configuration(str): The configuration object of the resource in valid JSON format. It must match the schema registered with AWS CloudFormation.  The configuration JSON must not exceed 64 KB. .
        tags(Dict, optional): Tags associated with the resource. Defaults to None.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.config.resource.config.init.put(
                ctx,
                resource_type=value,
                schema_version_id=value,
                resource_id=value,
                configuration=value,
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.config.resource.config.init.put resource_type=value, schema_version_id=value, resource_id=value, configuration=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="config",
        operation="put_resource_config",
        op_kwargs={
            "ResourceType": resource_type,
            "SchemaVersionId": schema_version_id,
            "ResourceId": resource_id,
            "ResourceName": resource_name,
            "Configuration": configuration,
            "Tags": tags,
        },
    )


async def select(
    hub, ctx, expression: str, limit: int = None, next_token: str = None
) -> Dict:
    r"""
    **Autogenerated function**

    Accepts a structured query language (SQL) SELECT command, performs the corresponding search, and returns resource configurations matching the properties. For more information about query components, see the  Query Components  section in the AWS Config Developer Guide.

    Args:
        expression(str): The SQL query SELECT command.
        limit(int, optional): The maximum number of query results returned on each page. . Defaults to None.
        next_token(str, optional): The nextToken string returned in a previous request that you use to request the next page of results in a paginated response. . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.config.resource.config.init.select(ctx, expression=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.config.resource.config.init.select expression=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="config",
        operation="select_resource_config",
        op_kwargs={"Expression": expression, "Limit": limit, "NextToken": next_token},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def create(
    hub,
    ctx,
    name: str,
    event_filter: Dict,
    event_bridge_bus: str,
    description: str = None,
    client_token: str = None,
    tags: Dict = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Creates an EventIntegration, given a specified name, description, and a reference to an Amazon EventBridge bus in your account and a partner event source that pushes events to that bus. No objects are created in the your account, only metadata that is persisted on the EventIntegration control plane.

    Args:
        name(str): The name of the event integration.
        description(str, optional): The description of the event integration. Defaults to None.
        event_filter(Dict): The event filter.
        event_bridge_bus(str): The EventBridge bus.
        client_token(str, optional): A unique, case-sensitive identifier that you provide to ensure the idempotency of the request. Defaults to None.
        tags(Dict, optional): One or more tags. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.appintegration.event_integration.init.create(
                ctx, name=value, event_filter=value, event_bridge_bus=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.appintegration.event_integration.init.create name=value, event_filter=value, event_bridge_bus=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="appintegrations",
        operation="create_event_integration",
        op_kwargs={
            "Name": name,
            "Description": description,
            "EventFilter": event_filter,
            "EventBridgeBus": event_bridge_bus,
            "ClientToken": client_token,
            "Tags": tags,
        },
    )


async def delete(hub, ctx, name: str) -> Dict:
    r"""
    **Autogenerated function**

    Deletes the specified existing event integration. If the event integration is associated with clients, the request is rejected.

    Args:
        name(str): The name of the event integration.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.appintegration.event_integration.init.delete(ctx, name=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.appintegration.event_integration.init.delete name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="appintegrations",
        operation="delete_event_integration",
        op_kwargs={"Name": name},
    )


async def get(hub, ctx, name: str) -> Dict:
    r"""
    **Autogenerated function**

    Return information about the event integration.

    Args:
        name(str): The name of the event integration. .

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.appintegration.event_integration.init.get(ctx, name=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.appintegration.event_integration.init.get name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="appintegrations",
        operation="get_event_integration",
        op_kwargs={"Name": name},
    )


async def list_all(hub, ctx, next_token: str = None, max_results: int = None) -> Dict:
    r"""
    **Autogenerated function**

    Returns a paginated list of event integrations in the account.

    Args:
        next_token(str, optional): The token for the next set of results. Use the value returned in the previous response in the next request to retrieve the next set of results. Defaults to None.
        max_results(int, optional): The maximum number of results to return per page. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.appintegration.event_integration.init.list_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.appintegration.event_integration.init.list_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="appintegrations",
        operation="list_event_integrations",
        op_kwargs={"NextToken": next_token, "MaxResults": max_results},
    )


async def update(hub, ctx, name: str, description: str = None) -> Dict:
    r"""
    **Autogenerated function**

    Updates the description of an event integration.

    Args:
        name(str): The name of the event integration.
        description(str, optional): The description of the event inegration. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.appintegration.event_integration.init.update(ctx, name=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.appintegration.event_integration.init.update name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="appintegrations",
        operation="update_event_integration",
        op_kwargs={"Name": name, "Description": description},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def send(
    hub,
    ctx,
    callback_token: str,
    failure_reason: str = None,
    client_request_token: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Notifies the pipeline that the execution of a callback step failed, along with a message describing why. When a callback step is run, the pipeline generates a callback token and includes the token in a message sent to Amazon Simple Queue Service (Amazon SQS).

    Args:
        callback_token(str): The pipeline generated token from the Amazon SQS queue.
        failure_reason(str, optional): A message describing why the step failed. Defaults to None.
        client_request_token(str, optional): A unique, case-sensitive identifier that you provide to ensure the idempotency of the operation. An idempotent operation completes no more than one time. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.sagemaker.pipeline.execution.step.failure.send(
                ctx, callback_token=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.sagemaker.pipeline.execution.step.failure.send callback_token=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="sagemaker",
        operation="send_pipeline_execution_step_failure",
        op_kwargs={
            "CallbackToken": callback_token,
            "FailureReason": failure_reason,
            "ClientRequestToken": client_request_token,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def describe(hub, ctx, next_token: str = None, max_results: int = None) -> Dict:
    r"""
    **Autogenerated function**

    Use this operation to see information from the most recent run of each canary that you have created.

    Args:
        next_token(str, optional): A token that indicates that there is more data available. You can use this token in a subsequent DescribeCanaries operation to retrieve the next set of results. Defaults to None.
        max_results(int, optional): Specify this parameter to limit how many runs are returned each time you use the DescribeLastRun operation. If you omit this parameter, the default of 100 is used. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.synthetic.canary.last_run.describe(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.synthetic.canary.last_run.describe
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="synthetics",
        operation="describe_canaries_last_run",
        op_kwargs={"NextToken": next_token, "MaxResults": max_results},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def delete(hub, ctx, backup_vault_name: str) -> None:
    r"""
    **Autogenerated function**

    Deletes the policy document that manages permissions on a backup vault.

    Args:
        backup_vault_name(str): The name of a logical container where backups are stored. Backup vaults are identified by names that are unique to the account used to create them and the AWS Region where they are created. They consist of lowercase letters, numbers, and hyphens.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.backup.backup.vault.access_policy.delete(
                ctx, backup_vault_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.backup.backup.vault.access_policy.delete backup_vault_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="backup",
        operation="delete_backup_vault_access_policy",
        op_kwargs={"BackupVaultName": backup_vault_name},
    )


async def get(hub, ctx, backup_vault_name: str) -> Dict:
    r"""
    **Autogenerated function**

    Returns the access policy document that is associated with the named backup vault.

    Args:
        backup_vault_name(str): The name of a logical container where backups are stored. Backup vaults are identified by names that are unique to the account used to create them and the AWS Region where they are created. They consist of lowercase letters, numbers, and hyphens.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.backup.backup.vault.access_policy.get(
                ctx, backup_vault_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.backup.backup.vault.access_policy.get backup_vault_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="backup",
        operation="get_backup_vault_access_policy",
        op_kwargs={"BackupVaultName": backup_vault_name},
    )


async def put(hub, ctx, backup_vault_name: str, policy: str = None) -> None:
    r"""
    **Autogenerated function**

    Sets a resource-based policy that is used to manage access permissions on the target backup vault. Requires a backup vault name and an access policy document in JSON format.

    Args:
        backup_vault_name(str): The name of a logical container where backups are stored. Backup vaults are identified by names that are unique to the account used to create them and the AWS Region where they are created. They consist of lowercase letters, numbers, and hyphens.
        policy(str, optional): The backup vault access policy document in JSON format. Defaults to None.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.backup.backup.vault.access_policy.put(
                ctx, backup_vault_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.backup.backup.vault.access_policy.put backup_vault_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="backup",
        operation="put_backup_vault_access_policy",
        op_kwargs={"BackupVaultName": backup_vault_name, "Policy": policy},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def create(hub, ctx, collection_id: str, tags: Dict = None) -> Dict:
    r"""
    **Autogenerated function**

    Creates a collection in an AWS Region. You can add faces to the collection using the IndexFaces operation.  For example, you might create collections, one for each of your application users. A user can then index faces using the IndexFaces operation and persist results in a specific collection. Then, a user can search the collection for faces in the user-specific container.  When you create a collection, it is associated with the latest version of the face model version.  Collection names are case-sensitive.  This operation requires permissions to perform the rekognition:CreateCollection action. If you want to tag your collection, you also require permission to perform the rekognition:TagResource operation.

    Args:
        collection_id(str): ID for the collection that you are creating.
        tags(Dict, optional):  A set of tags (key-value pairs) that you want to attach to the collection. . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.rekognition.collection.create(ctx, collection_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.rekognition.collection.create collection_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="rekognition",
        operation="create_collection",
        op_kwargs={"CollectionId": collection_id, "Tags": tags},
    )


async def delete(hub, ctx, collection_id: str) -> Dict:
    r"""
    **Autogenerated function**

    Deletes the specified collection. Note that this operation removes all faces in the collection. For an example, see delete-collection-procedure. This operation requires permissions to perform the rekognition:DeleteCollection action.

    Args:
        collection_id(str): ID of the collection to delete.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.rekognition.collection.delete(ctx, collection_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.rekognition.collection.delete collection_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="rekognition",
        operation="delete_collection",
        op_kwargs={"CollectionId": collection_id},
    )


async def describe(hub, ctx, collection_id: str) -> Dict:
    r"""
    **Autogenerated function**

    Describes the specified collection. You can use DescribeCollection to get information, such as the number of faces indexed into a collection and the version of the model used by the collection for face detection. For more information, see Describing a Collection in the Amazon Rekognition Developer Guide.

    Args:
        collection_id(str): The ID of the collection to describe.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.rekognition.collection.describe(ctx, collection_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.rekognition.collection.describe collection_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="rekognition",
        operation="describe_collection",
        op_kwargs={"CollectionId": collection_id},
    )


async def list_all(hub, ctx, next_token: str = None, max_results: int = None) -> Dict:
    r"""
    **Autogenerated function**

    Returns list of collection IDs in your account. If the result is truncated, the response also provides a NextToken that you can use in the subsequent request to fetch the next set of collection IDs. For an example, see Listing Collections in the Amazon Rekognition Developer Guide. This operation requires permissions to perform the rekognition:ListCollections action.

    Args:
        next_token(str, optional): Pagination token from the previous response. Defaults to None.
        max_results(int, optional): Maximum number of collection IDs to return. . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.rekognition.collection.list_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.rekognition.collection.list_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="rekognition",
        operation="list_collections",
        op_kwargs={"NextToken": next_token, "MaxResults": max_results},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def describe_all(
    hub,
    ctx,
    organization_config_rule_names: List = None,
    limit: int = None,
    next_token: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Provides organization config rule deployment status for an organization.  The status is not considered successful until organization config rule is successfully deployed in all the member accounts with an exception of excluded accounts. When you specify the limit and the next token, you receive a paginated response. Limit and next token are not applicable if you specify organization config rule names. It is only applicable, when you request all the organization config rules.

    Args:
        organization_config_rule_names(List, optional): The names of organization config rules for which you want status details. If you do not specify any names, AWS Config returns details for all your organization AWS Confg rules. Defaults to None.
        limit(int, optional): The maximum number of OrganizationConfigRuleStatuses returned on each page. If you do no specify a number, AWS Config uses the default. The default is 100. Defaults to None.
        next_token(str, optional): The nextToken string returned on a previous page that you use to get the next page of results in a paginated response. . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.config.organization.config_rule.status.describe_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.config.organization.config_rule.status.describe_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="config",
        operation="describe_organization_config_rule_statuses",
        op_kwargs={
            "OrganizationConfigRuleNames": organization_config_rule_names,
            "Limit": limit,
            "NextToken": next_token,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def describe(hub, ctx, registration_token: str) -> Dict:
    r"""
    **Autogenerated function**

    Returns information about an extension's registration, including its current status and type and version identifiers. When you initiate a registration request using  RegisterType , you can then use  DescribeTypeRegistration  to monitor the progress of that registration request. Once the registration request has completed, use  DescribeType  to return detailed information about an extension.

    Args:
        registration_token(str): The identifier for this registration request. This registration token is generated by CloudFormation when you initiate a registration request using  RegisterType .

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cloudformation.type_.registration.describe(
                ctx, registration_token=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cloudformation.type_.registration.describe registration_token=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cloudformation",
        operation="describe_type_registration",
        op_kwargs={"RegistrationToken": registration_token},
    )


async def list_all(
    hub,
    ctx,
    type_: str = None,
    type_name: str = None,
    type_arn: str = None,
    registration_status_filter: str = None,
    max_results: int = None,
    next_token: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Returns a list of registration tokens for the specified extension(s).

    Args:
        type_(str, optional): The kind of extension. Conditional: You must specify either TypeName and Type, or Arn. Defaults to None.
        type_name(str, optional): The name of the extension. Conditional: You must specify either TypeName and Type, or Arn. Defaults to None.
        type_arn(str, optional): The Amazon Resource Name (ARN) of the extension. Conditional: You must specify either TypeName and Type, or Arn. Defaults to None.
        registration_status_filter(str, optional): The current status of the extension registration request. The default is IN_PROGRESS. Defaults to None.
        max_results(int, optional): The maximum number of results to be returned with a single call. If the number of available results exceeds this maximum, the response includes a NextToken value that you can assign to the NextToken request parameter to get the next set of results. Defaults to None.
        next_token(str, optional): If the previous paginated request didn't return all of the remaining results, the response object's NextToken parameter value is set to a token. To retrieve the next set of results, call this action again and assign that token to the request object's NextToken parameter. If there are no remaining results, the previous response object's NextToken parameter is set to null. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cloudformation.type_.registration.list_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cloudformation.type_.registration.list_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cloudformation",
        operation="list_type_registrations",
        op_kwargs={
            "Type": type_,
            "TypeName": type_name,
            "TypeArn": type_arn,
            "RegistrationStatusFilter": registration_status_filter,
            "MaxResults": max_results,
            "NextToken": next_token,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def get(
    hub,
    ctx,
    domain: str,
    execution: Dict,
    next_page_token: str = None,
    maximum_page_size: int = None,
    reverse_order: bool = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Returns the history of the specified workflow execution. The results may be split into multiple pages. To retrieve subsequent pages, make the call again using the nextPageToken returned by the initial call.  This operation is eventually consistent. The results are best effort and may not exactly reflect recent updates and changes.   Access Control  You can use IAM policies to control this action's access to Amazon SWF resources as follows:   Use a Resource element with the domain name to limit the action to only specified domains.   Use an Action element to allow or deny permission to call this action.   You cannot use an IAM policy to constrain this action's parameters.   If the caller doesn't have sufficient permissions to invoke the action, or the parameter values fall outside the specified constraints, the action fails. The associated event attribute's cause parameter is set to OPERATION_NOT_PERMITTED. For details and example IAM policies, see Using IAM to Manage Access to Amazon SWF Workflows in the Amazon SWF Developer Guide.

    Args:
        domain(str): The name of the domain containing the workflow execution.
        execution(Dict): Specifies the workflow execution for which to return the history.
        next_page_token(str, optional): If NextPageToken is returned there are more results available. The value of NextPageToken is a unique pagination token for each page. Make the call again using the returned token to retrieve the next page. Keep all other arguments unchanged. Each pagination token expires after 60 seconds. Using an expired pagination token will return a 400 error: "Specified token has exceeded its maximum lifetime".  The configured maximumPageSize determines how many results can be returned in a single call. . Defaults to None.
        maximum_page_size(int, optional): The maximum number of results that are returned per call. Use nextPageToken to obtain further pages of results. . Defaults to None.
        reverse_order(bool, optional): When set to true, returns the events in reverse order. By default the results are returned in ascending order of the eventTimeStamp of the events. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.swf.workflow.execution.history.get(
                ctx, domain=value, execution=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.swf.workflow.execution.history.get domain=value, execution=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="swf",
        operation="get_workflow_execution_history",
        op_kwargs={
            "domain": domain,
            "execution": execution,
            "nextPageToken": next_page_token,
            "maximumPageSize": maximum_page_size,
            "reverseOrder": reverse_order,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def create(
    hub,
    ctx,
    api_id: str,
    name: str,
    schema: str,
    content_type: str = None,
    description: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Creates a Model for an API.

    Args:
        api_id(str): The API identifier.
        content_type(str, optional): The content-type for the model, for example, "application/json". Defaults to None.
        description(str, optional): The description of the model. Defaults to None.
        name(str): The name of the model. Must be alphanumeric.
        schema(str): The schema for the model. For application/json models, this should be JSON schema draft 4 model.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.apigatewayv2.model.init.create(
                ctx, api_id=value, name=value, schema=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.apigatewayv2.model.init.create api_id=value, name=value, schema=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="apigatewayv2",
        operation="create_model",
        op_kwargs={
            "ApiId": api_id,
            "ContentType": content_type,
            "Description": description,
            "Name": name,
            "Schema": schema,
        },
    )


async def delete(hub, ctx, api_id: str, model_id: str) -> None:
    r"""
    **Autogenerated function**

    Deletes a Model.

    Args:
        api_id(str): The API identifier.
        model_id(str): The model ID.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.apigatewayv2.model.init.delete(
                ctx, api_id=value, model_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.apigatewayv2.model.init.delete api_id=value, model_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="apigatewayv2",
        operation="delete_model",
        op_kwargs={"ApiId": api_id, "ModelId": model_id},
    )


async def get(hub, ctx, api_id: str, model_id: str) -> Dict:
    r"""
    **Autogenerated function**

    Gets a Model.

    Args:
        api_id(str): The API identifier.
        model_id(str): The model ID.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.apigatewayv2.model.init.get(ctx, api_id=value, model_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.apigatewayv2.model.init.get api_id=value, model_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="apigatewayv2",
        operation="get_model",
        op_kwargs={"ApiId": api_id, "ModelId": model_id},
    )


async def get_all(
    hub, ctx, api_id: str, max_results: str = None, next_token: str = None
) -> Dict:
    r"""
    **Autogenerated function**

    Gets the Models for an API.

    Args:
        api_id(str): The API identifier.
        max_results(str, optional): The maximum number of elements to be returned for this resource. Defaults to None.
        next_token(str, optional): The next page of elements from this collection. Not valid for the last element of the collection. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.apigatewayv2.model.init.get_all(ctx, api_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.apigatewayv2.model.init.get_all api_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="apigatewayv2",
        operation="get_models",
        op_kwargs={"ApiId": api_id, "MaxResults": max_results, "NextToken": next_token},
    )


async def update(
    hub,
    ctx,
    api_id: str,
    model_id: str,
    content_type: str = None,
    description: str = None,
    name: str = None,
    schema: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Updates a Model.

    Args:
        api_id(str): The API identifier.
        content_type(str, optional): The content-type for the model, for example, "application/json". Defaults to None.
        description(str, optional): The description of the model. Defaults to None.
        model_id(str): The model ID.
        name(str, optional): The name of the model. Defaults to None.
        schema(str, optional): The schema for the model. For application/json models, this should be JSON schema draft 4 model. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.apigatewayv2.model.init.update(
                ctx, api_id=value, model_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.apigatewayv2.model.init.update api_id=value, model_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="apigatewayv2",
        operation="update_model",
        op_kwargs={
            "ApiId": api_id,
            "ContentType": content_type,
            "Description": description,
            "ModelId": model_id,
            "Name": name,
            "Schema": schema,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


__func_alias__ = {"list_": "list"}


async def list_(
    hub, ctx, resource_id: str, max_results: int = None, next_token: str = None
) -> Dict:
    r"""
    **Autogenerated function**

    Lists all tags for a top-level EFS resource. You must provide the ID of the resource that you want to retrieve the tags for. This operation requires permissions for the elasticfilesystem:DescribeAccessPoints action.

    Args:
        resource_id(str): Specifies the EFS resource you want to retrieve tags for. You can retrieve tags for EFS file systems and access points using this API endpoint.
        max_results(int, optional): (Optional) Specifies the maximum number of tag objects to return in the response. The default value is 100. Defaults to None.
        next_token(str, optional): (Optional) You can use NextToken in a subsequent request to fetch the next page of access point descriptions if the response payload was paginated. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.efs.tags.for_resource.list(ctx, resource_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.efs.tags.for_resource.list resource_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="efs",
        operation="list_tags_for_resource",
        op_kwargs={
            "ResourceId": resource_id,
            "MaxResults": max_results,
            "NextToken": next_token,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def get_all(hub, ctx, file_name: str, map_name: str) -> Dict:
    r"""
    **Autogenerated function**

    Retrieves the sprite sheet corresponding to a map resource. The sprite sheet is a PNG image paired with a JSON document describing the offsets of individual icons that will be displayed on a rendered map.

    Args:
        file_name(str): The name of the sprite ﬁle. Use the following ﬁle names for the sprite sheet:    sprites.png     sprites@2x.png for high pixel density displays   For the JSON document contain image offsets. Use the following ﬁle names:    sprites.json     sprites@2x.json for high pixel density displays  .
        map_name(str): The map resource associated with the sprite ﬁle.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.location.map_.sprite.get_all(
                ctx, file_name=value, map_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.location.map_.sprite.get_all file_name=value, map_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="location",
        operation="get_map_sprites",
        op_kwargs={"FileName": file_name, "MapName": map_name},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def get(hub, ctx, bucket: str, expected_bucket_owner: str = None) -> Dict:
    r"""
    **Autogenerated function**

    Returns the logging status of a bucket and the permissions users have to view and modify that status. To use GET, you must be the bucket owner. The following operations are related to GetBucketLogging:    CreateBucket     PutBucketLogging

    Args:
        bucket(str): The bucket name for which to get the logging information.
        expected_bucket_owner(str, optional): The account ID of the expected bucket owner. If the bucket is owned by a different account, the request will fail with an HTTP 403 (Access Denied) error. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.s3.bucket.logging.get(ctx, bucket=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.s3.bucket.logging.get bucket=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="s3",
        operation="get_bucket_logging",
        op_kwargs={"Bucket": bucket, "ExpectedBucketOwner": expected_bucket_owner},
    )


async def put(
    hub,
    ctx,
    bucket: str,
    bucket_logging_status: Dict,
    content_md5: str = None,
    expected_bucket_owner: str = None,
) -> None:
    r"""
    **Autogenerated function**

    Set the logging parameters for a bucket and to specify permissions for who can view and modify the logging parameters. All logs are saved to buckets in the same AWS Region as the source bucket. To set the logging status of a bucket, you must be the bucket owner. The bucket owner is automatically granted FULL_CONTROL to all logs. You use the Grantee request element to grant access to other people. The Permissions request element specifies the kind of access the grantee has to the logs.  Grantee Values  You can specify the person (grantee) to whom you're assigning access rights (using request elements) in the following ways:   By the person's ID:  <Grantee xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:type="CanonicalUser"><ID><>ID<></ID><DisplayName><>GranteesEmail<></DisplayName> </Grantee>  DisplayName is optional and ignored in the request.   By Email address:   <Grantee xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:type="AmazonCustomerByEmail"><EmailAddress><>Grantees@email.com<></EmailAddress></Grantee>  The grantee is resolved to the CanonicalUser and, in a response to a GET Object acl request, appears as the CanonicalUser.   By URI:  <Grantee xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:type="Group"><URI><>http://acs.amazonaws.com/groups/global/AuthenticatedUsers<></URI></Grantee>    To enable logging, you use LoggingEnabled and its children request elements. To disable logging, you use an empty BucketLoggingStatus request element:  <BucketLoggingStatus xmlns="http://doc.s3.amazonaws.com/2006-03-01" />  For more information about server access logging, see Server Access Logging.  For more information about creating a bucket, see CreateBucket. For more information about returning the logging status of a bucket, see GetBucketLogging. The following operations are related to PutBucketLogging:    PutObject     DeleteBucket     CreateBucket     GetBucketLogging

    Args:
        bucket(str): The name of the bucket for which to set the logging parameters.
        bucket_logging_status(Dict): Container for logging status information.
        content_md5(str, optional): The MD5 hash of the PutBucketLogging request body. For requests made using the AWS Command Line Interface (CLI) or AWS SDKs, this field is calculated automatically. Defaults to None.
        expected_bucket_owner(str, optional): The account ID of the expected bucket owner. If the bucket is owned by a different account, the request will fail with an HTTP 403 (Access Denied) error. Defaults to None.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.s3.bucket.logging.put(
                ctx, bucket=value, bucket_logging_status=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.s3.bucket.logging.put bucket=value, bucket_logging_status=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="s3",
        operation="put_bucket_logging",
        op_kwargs={
            "Bucket": bucket,
            "BucketLoggingStatus": bucket_logging_status,
            "ContentMD5": content_md5,
            "ExpectedBucketOwner": expected_bucket_owner,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def get(hub, ctx, domain_name: str) -> Dict:
    r"""
    **Autogenerated function**

    This operation returns detailed information about a specified domain that is associated with the current AWS account. Contact information for the domain is also returned as part of the output.

    Args:
        domain_name(str): The name of the domain that you want to get detailed information about.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.route53domain.domain.detail.get(ctx, domain_name=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.route53domain.domain.detail.get domain_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="route53domains",
        operation="get_domain_detail",
        op_kwargs={"DomainName": domain_name},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def describe(
    hub, ctx, configuration_id: str, configuration_revision: str
) -> Dict:
    r"""
    **Autogenerated function**

    Returns the specified configuration revision for the specified configuration.

    Args:
        configuration_id(str): The unique ID that Amazon MQ generates for the configuration.
        configuration_revision(str): The revision of the configuration.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.mq.configuration.revision.describe(
                ctx, configuration_id=value, configuration_revision=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.mq.configuration.revision.describe configuration_id=value, configuration_revision=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="mq",
        operation="describe_configuration_revision",
        op_kwargs={
            "ConfigurationId": configuration_id,
            "ConfigurationRevision": configuration_revision,
        },
    )


async def list_all(
    hub, ctx, configuration_id: str, max_results: int = None, next_token: str = None
) -> Dict:
    r"""
    **Autogenerated function**

    Returns a list of all revisions for the specified configuration.

    Args:
        configuration_id(str): The unique ID that Amazon MQ generates for the configuration.
        max_results(int, optional): The maximum number of brokers that Amazon MQ can return per page (20 by default). This value must be an integer from 5 to 100. Defaults to None.
        next_token(str, optional): The token that specifies the next page of results Amazon MQ should return. To request the first page, leave nextToken empty. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.mq.configuration.revision.list_all(ctx, configuration_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.mq.configuration.revision.list_all configuration_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="mq",
        operation="list_configuration_revisions",
        op_kwargs={
            "ConfigurationId": configuration_id,
            "MaxResults": max_results,
            "NextToken": next_token,
        },
    )
